@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-08 by the Quassel Project                          *
+ *   Copyright (C) 2005-09 by the Quassel Project                          *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -37,13 +37,27 @@
 *****************************************/
 NetworkItem::NetworkItem(const NetworkId &netid, AbstractTreeItem *parent)
   : PropertyMapItem(QList<QString>() << "networkName" << "currentServer" << "nickCount", parent),
-    _networkId(netid)
+    _networkId(netid),
+    _statusBufferItem(0)
 {
-  setFlags(Qt::ItemIsEnabled);
+  // DO NOT EMIT dataChanged() DIRECTLY IN NetworkItem
+  // use networkDataChanged() instead. Otherwise you will end up in a infinite loop
+  // as we "sync" the dataChanged() signals of NetworkItem and StatusBufferItem
+  setFlags(Qt::ItemIsEnabled | Qt::ItemIsSelectable);
+  connect(this, SIGNAL(networkDataChanged(int)), this, SIGNAL(dataChanged(int)));
+  connect(this, SIGNAL(beginRemoveChilds(int, int)), this, SLOT(onBeginRemoveChilds(int, int)));
 }
 
 QVariant NetworkItem::data(int column, int role) const {
   switch(role) {
+  case NetworkModel::BufferIdRole:
+  case NetworkModel::BufferInfoRole:
+  case NetworkModel::BufferTypeRole:
+  case NetworkModel::BufferActivityRole:
+    if(_statusBufferItem)
+      return _statusBufferItem->data(column, role);
+    else
+      return QVariant();
   case NetworkModel::NetworkIdRole:
     return qVariantFromValue(_networkId);
   case NetworkModel::ItemTypeRole:
@@ -57,7 +71,7 @@ QVariant NetworkItem::data(int column, int role) const {
 
 BufferItem *NetworkItem::findBufferItem(BufferId bufferId) {
   BufferItem *bufferItem = 0;
-  
+
   for(int i = 0; i < childCount(); i++) {
     bufferItem = qobject_cast<BufferItem *>(child(i));
     if(!bufferItem)
@@ -73,10 +87,14 @@ BufferItem *NetworkItem::bufferItem(const BufferInfo &bufferInfo) {
   BufferItem *bufferItem = findBufferItem(bufferInfo);
   if(bufferItem)
     return bufferItem;
-  
+
   switch(bufferInfo.type()) {
   case BufferInfo::StatusBuffer:
-    bufferItem = new StatusBufferItem(bufferInfo, this);
+    _statusBufferItem = new StatusBufferItem(bufferInfo, this);
+    bufferItem = _statusBufferItem;
+    disconnect(this, SIGNAL(networkDataChanged(int)), this, SIGNAL(dataChanged(int)));
+    connect(this, SIGNAL(networkDataChanged(int)), bufferItem, SIGNAL(dataChanged(int)));
+    connect(bufferItem, SIGNAL(dataChanged(int)), this, SIGNAL(dataChanged(int)));
     break;
   case BufferInfo::ChannelBuffer:
     bufferItem = new ChannelBufferItem(bufferInfo, this);
@@ -95,7 +113,7 @@ BufferItem *NetworkItem::bufferItem(const BufferInfo &bufferInfo) {
 void NetworkItem::attachNetwork(Network *network) {
   if(!network)
     return;
-  
+
   _network = network;
 
   connect(network, SIGNAL(networkNameSet(QString)),
@@ -107,11 +125,11 @@ void NetworkItem::attachNetwork(Network *network) {
   connect(network, SIGNAL(ircUserAdded(IrcUser *)),
 	  this, SLOT(attachIrcUser(IrcUser *)));
   connect(network, SIGNAL(connectedSet(bool)),
-	  this, SIGNAL(dataChanged()));
+	  this, SIGNAL(networkDataChanged()));
   connect(network, SIGNAL(destroyed()),
-	  this, SIGNAL(dataChanged()));
-  
-  emit dataChanged();
+	  this, SIGNAL(networkDataChanged()));
+
+  emit networkDataChanged();
 }
 
 void NetworkItem::attachIrcChannel(IrcChannel *ircChannel) {
@@ -136,7 +154,7 @@ void NetworkItem::attachIrcUser(IrcUser *ircUser) {
       continue;
 
     if(queryItem->bufferName().toLower() == ircUser->nick().toLower()) {
-      queryItem->attachIrcUser(ircUser);
+      queryItem->setIrcUser(ircUser);
       break;
     }
   }
@@ -144,12 +162,12 @@ void NetworkItem::attachIrcUser(IrcUser *ircUser) {
 
 void NetworkItem::setNetworkName(const QString &networkName) {
   Q_UNUSED(networkName);
-  emit dataChanged(0);
+  emit networkDataChanged(0);
 }
 
 void NetworkItem::setCurrentServer(const QString &serverName) {
   Q_UNUSED(serverName);
-  emit dataChanged(1);
+  emit networkDataChanged(1);
 }
 
 
@@ -167,6 +185,16 @@ QString NetworkItem::toolTip(int column) const {
   return QString("<p> %1 </p>").arg(toolTip.join("<br />"));
 }
 
+void NetworkItem::onBeginRemoveChilds(int start, int end) {
+  for(int i = start; i <= end; i++) {
+    StatusBufferItem *statusBufferItem = qobject_cast<StatusBufferItem *>(child(i));
+    if(statusBufferItem) {
+      _statusBufferItem = 0;
+      break;
+    }
+  }
+}
+
 /*****************************************
 *  Fancy Buffer Items
 *****************************************/
@@ -185,9 +213,16 @@ void BufferItem::setActivityLevel(BufferInfo::ActivityLevel level) {
   }
 }
 
+void BufferItem::clearActivityLevel() {
+  _activity = BufferInfo::NoActivity;
+  _lastSeenMarkerMsgId = _lastSeenMsgId;
+  emit dataChanged();
+}
+
 void BufferItem::updateActivityLevel(const Message &msg) {
-  if(isCurrentBuffer())
+  if(isCurrentBuffer()) {
     return;
+  }
 
   if(msg.flags() & Message::Self)	// don't update activity for our own messages
     return;
@@ -230,7 +265,6 @@ QVariant BufferItem::data(int column, int role) const {
 }
 
 bool BufferItem::setData(int column, const QVariant &value, int role) {
-  qDebug() << "BufferItem::setData(int column, const QVariant &value, int role):" << this << column << value << role;
   switch(role) {
   case NetworkModel::BufferActivityRole:
     setActivityLevel((BufferInfo::ActivityLevel)value.toInt());
@@ -248,7 +282,10 @@ void BufferItem::setBufferName(const QString &name) {
 
 void BufferItem::setLastSeenMsgId(const MsgId &msgId) {
   _lastSeenMsgId = msgId;
-  setActivityLevel(BufferInfo::NoActivity); 
+  if(!isCurrentBuffer()) {
+    _lastSeenMarkerMsgId = msgId;
+  }
+  setActivityLevel(BufferInfo::NoActivity);
 }
 
 bool BufferItem::isCurrentBuffer() const {
@@ -266,18 +303,14 @@ QString BufferItem::toolTip(int column) const {
 StatusBufferItem::StatusBufferItem(const BufferInfo &bufferInfo, NetworkItem *parent)
   : BufferItem(bufferInfo, parent)
 {
-  Q_ASSERT(parent);
-  connect(parent, SIGNAL(dataChanged()), this, SIGNAL(dataChanged()));
 }
 
 QString StatusBufferItem::toolTip(int column) const {
-  Q_UNUSED(column);
-  QStringList toolTip;
-
-  QString netName = Client::network(bufferInfo().networkId())->networkName();
-  toolTip.append(tr("<b>Status buffer of %1</b>").arg(netName));
-
-  return tr("<p> %1 </p>").arg(toolTip.join("<br />"));
+  NetworkItem *networkItem = qobject_cast<NetworkItem *>(parent());
+  if(networkItem)
+    return networkItem->toolTip(column);
+  else
+    return QString();
 }
 
 /*****************************************
@@ -287,19 +320,22 @@ QueryBufferItem::QueryBufferItem(const BufferInfo &bufferInfo, NetworkItem *pare
   : BufferItem(bufferInfo, parent),
     _ircUser(0)
 {
-  setFlags(flags() | Qt::ItemIsDropEnabled);
+  setFlags(flags() | Qt::ItemIsDropEnabled | Qt::ItemIsEditable);
 
   const Network *net = Client::network(bufferInfo.networkId());
   if(!net)
     return;
 
   IrcUser *ircUser = net->ircUser(bufferInfo.bufferName());
-  if(ircUser)
-    attachIrcUser(ircUser);
+  setIrcUser(ircUser);
 }
 
 QVariant QueryBufferItem::data(int column, int role) const {
   switch(role) {
+  case Qt::EditRole:
+    return BufferItem::data(column, Qt::DisplayRole);
+  case NetworkModel::IrcUserRole:
+    return QVariant::fromValue<QObject *>(_ircUser);
   case NetworkModel::UserAwayRole:
     return (bool)_ircUser ? _ircUser->isAway() : false;
   default:
@@ -307,8 +343,37 @@ QVariant QueryBufferItem::data(int column, int role) const {
   }
 }
 
+bool QueryBufferItem::setData(int column, const QVariant &value, int role) {
+  if(column != 0)
+    return BufferItem::setData(column, value, role);
+
+  switch(role) {
+  case Qt::EditRole:
+    {
+      QString newName = value.toString();
+      if(!newName.isEmpty()) {
+	Client::renameBuffer(bufferId(), newName);
+	return true;
+      } else {
+	return false;
+      }
+    }
+    break;
+  default:
+    return BufferItem::setData(column, value, role);
+  }
+}
+
+void QueryBufferItem::setBufferName(const QString &name) {
+  BufferItem::setBufferName(name);
+  NetworkId netId = data(0, NetworkModel::NetworkIdRole).value<NetworkId>();
+  const Network *net = Client::network(netId);
+  if(net)
+    setIrcUser(net->ircUser(name));
+}
+
 QString QueryBufferItem::toolTip(int column) const {
-  // pretty much code duplication of IrcUserItem::toolTip() but inheritance won't solve this... 
+  // pretty much code duplication of IrcUserItem::toolTip() but inheritance won't solve this...
   Q_UNUSED(column);
   QStringList toolTip;
 
@@ -323,9 +388,9 @@ QString QueryBufferItem::toolTip(int column) const {
     if(!_ircUser->ircOperator().isEmpty()) toolTip.append(QString("%1 %2").arg(_ircUser->nick()).arg(_ircUser->ircOperator()));
     if(!_ircUser->suserHost().isEmpty()) toolTip.append(_ircUser->suserHost());
     if(!_ircUser->whoisServiceReply().isEmpty()) toolTip.append(_ircUser->whoisServiceReply());
-    
+
     toolTip.append(_ircUser->hostmask().remove(0, _ircUser->hostmask().indexOf("!")+1));
-    
+
     if(_ircUser->idleTime().isValid()) {
       QDateTime now = QDateTime::currentDateTime();
       QDateTime idle = _ircUser->idleTime();
@@ -335,21 +400,31 @@ QString QueryBufferItem::toolTip(int column) const {
     if(_ircUser->loginTime().isValid()) {
       toolTip.append(tr("login time: %1").arg(_ircUser->loginTime().toString()));
     }
-    
+
     if(!_ircUser->server().isEmpty()) toolTip.append(tr("server: %1").arg(_ircUser->server()));
   }
 
   return QString("<p> %1 </p>").arg(toolTip.join("<br />"));
 }
 
-void QueryBufferItem::attachIrcUser(IrcUser *ircUser) {
+void QueryBufferItem::setIrcUser(IrcUser *ircUser) {
+  if(_ircUser == ircUser)
+    return;
+
+  if(_ircUser) {
+    disconnect(_ircUser, 0, this, 0);
+  }
+
+  if(ircUser) {
+    connect(ircUser, SIGNAL(quited()), this, SLOT(removeIrcUser()));
+    connect(ircUser, SIGNAL(awaySet(bool)), this, SIGNAL(dataChanged()));
+  }
+
   _ircUser = ircUser;
-  connect(_ircUser, SIGNAL(destroyed()), this, SLOT(ircUserDestroyed()));
-  connect(_ircUser, SIGNAL(awaySet(bool)), this, SIGNAL(dataChanged()));
   emit dataChanged();
 }
 
-void QueryBufferItem::ircUserDestroyed() {
+void QueryBufferItem::removeIrcUser() {
   _ircUser = 0;
   emit dataChanged();
 }
@@ -370,41 +445,51 @@ ChannelBufferItem::ChannelBufferItem(const BufferInfo &bufferInfo, AbstractTreeI
     attachIrcChannel(ircChannel);
 }
 
+QVariant ChannelBufferItem::data(int column, int role) const {
+  switch(role) {
+    case NetworkModel::IrcChannelRole:
+      return QVariant::fromValue<QObject *>(_ircChannel);
+    default:
+      return BufferItem::data(column, role);
+  }
+}
+
 QString ChannelBufferItem::toolTip(int column) const {
   Q_UNUSED(column);
   QStringList toolTip;
 
   toolTip.append(tr("<b>Channel %1</b>").arg(bufferName()));
   if(isActive()) {
-    //TODO: add channel modes 
+    //TODO: add channel modes
     toolTip.append(tr("<b>Users:</b> %1").arg(nickCount()));
     if(_ircChannel) {
       QString channelMode = _ircChannel->channelModeString(); // channelModeString is compiled on the fly -> thus cache the result
       if(!channelMode.isEmpty())
-	toolTip.append(tr("<b>Mode:</b> %1").arg(channelMode));
+        toolTip.append(tr("<b>Mode:</b> %1").arg(channelMode));
     }
-    
+
     BufferSettings s;
     bool showTopic = s.value("DisplayTopicInTooltip", QVariant(false)).toBool();
     if(showTopic) {
       QString _topic = topic();
       if(_topic != "") {
-	_topic.replace(QString("<"), QString("&lt;"));
-	_topic.replace(QString(">"), QString("&gt;"));
-	toolTip.append(QString("<font size='-2'>&nbsp;</font>"));
-	toolTip.append(tr("<b>Topic:</b> %1").arg(_topic));
+        _topic = stripFormatCodes(_topic);
+        _topic.replace(QString("<"), QString("&lt;"));
+        _topic.replace(QString(">"), QString("&gt;"));
+        toolTip.append(QString("<font size='-2'>&nbsp;</font>"));
+        toolTip.append(tr("<b>Topic:</b> %1").arg(_topic));
       }
     }
   } else {
     toolTip.append(tr("Not active <br /> Double-click to join"));
   }
 
-  return tr("<p> %1 </p>").arg(toolTip.join("<br />"));  
+  return tr("<p> %1 </p>").arg(toolTip.join("<br />"));
 }
 
 void ChannelBufferItem::attachIrcChannel(IrcChannel *ircChannel) {
   Q_ASSERT(!_ircChannel && ircChannel);
-  
+
   _ircChannel = ircChannel;
 
   connect(ircChannel, SIGNAL(topicSet(QString)),
@@ -413,8 +498,8 @@ void ChannelBufferItem::attachIrcChannel(IrcChannel *ircChannel) {
 	  this, SLOT(join(QList<IrcUser *>)));
   connect(ircChannel, SIGNAL(ircUserParted(IrcUser *)),
 	  this, SLOT(part(IrcUser *)));
-  connect(ircChannel, SIGNAL(destroyed()),
-	  this, SLOT(ircChannelDestroyed()));
+  connect(ircChannel, SIGNAL(parted()),
+	  this, SLOT(ircChannelParted()));
   connect(ircChannel, SIGNAL(ircUserModesSet(IrcUser *, QString)),
 	  this, SLOT(userModeChanged(IrcUser *)));
   connect(ircChannel, SIGNAL(ircUserModeAdded(IrcUser *, QString)),
@@ -424,11 +509,11 @@ void ChannelBufferItem::attachIrcChannel(IrcChannel *ircChannel) {
 
   if(!ircChannel->ircUsers().isEmpty())
     join(ircChannel->ircUsers());
-  
+
   emit dataChanged();
 }
 
-void ChannelBufferItem::ircChannelDestroyed() {
+void ChannelBufferItem::ircChannelParted() {
   Q_CHECK_PTR(_ircChannel);
   disconnect(_ircChannel, 0, this, 0);
   _ircChannel = 0;
@@ -443,7 +528,7 @@ void ChannelBufferItem::join(const QList<IrcUser *> &ircUsers) {
 
 UserCategoryItem *ChannelBufferItem::findCategoryItem(int categoryId) {
   UserCategoryItem *categoryItem = 0;
-  
+
   for(int i = 0; i < childCount(); i++) {
     categoryItem = qobject_cast<UserCategoryItem *>(child(i));
     if(!categoryItem)
@@ -465,7 +550,7 @@ void ChannelBufferItem::addUsersToCategory(const QList<IrcUser *> &ircUsers) {
 
   int categoryId = -1;
   UserCategoryItem *categoryItem = 0;
-  
+
   foreach(IrcUser *ircUser, ircUsers) {
     categoryId = UserCategoryItem::categoryFromModes(_ircChannel->userModes(ircUser));
     categoryItem = findCategoryItem(categoryId);
@@ -519,7 +604,7 @@ void ChannelBufferItem::userModeChanged(IrcUser *ircUser) {
 
   int categoryId = UserCategoryItem::categoryFromModes(_ircChannel->userModes(ircUser));
   UserCategoryItem *categoryItem = findCategoryItem(categoryId);
-    
+
   if(categoryItem) {
     if(categoryItem->findIrcUser(ircUser)) {
       return; // already in the right category;
@@ -559,6 +644,7 @@ UserCategoryItem::UserCategoryItem(int category, AbstractTreeItem *parent)
   : PropertyMapItem(QStringList() << "categoryName", parent),
     _category(category)
 {
+  setFlags(Qt::ItemIsEnabled);
   setTreeItemFlags(AbstractTreeItem::DeleteOnLastChildRemoved);
   setObjectName(parent->data(0, Qt::DisplayRole).toString() + "/" + QString::number(category));
 }
@@ -642,8 +728,8 @@ IrcUserItem::IrcUserItem(IrcUser *ircUser, AbstractTreeItem *parent)
   : PropertyMapItem(QStringList() << "nickName", parent),
     _ircUser(ircUser)
 {
-  setObjectName(ircUser->nick());  
-  connect(ircUser, SIGNAL(destroyed()), this, SLOT(ircUserDestroyed()));
+  setObjectName(ircUser->nick());
+  connect(ircUser, SIGNAL(quited()), this, SLOT(ircUserQuited()));
   connect(ircUser, SIGNAL(nickSet(QString)), this, SIGNAL(dataChanged()));
   connect(ircUser, SIGNAL(awaySet(bool)), this, SIGNAL(dataChanged()));
 }
@@ -660,6 +746,12 @@ QVariant IrcUserItem::data(int column, int role) const {
     return parent()->data(column, role);
   case NetworkModel::BufferInfoRole:
     return parent()->data(column, role);
+  case NetworkModel::IrcChannelRole:
+    return parent()->data(column, role);
+  case NetworkModel::IrcUserRole:
+    return QVariant::fromValue<QObject *>(_ircUser.data());
+  case NetworkModel::UserAwayRole:
+    return (bool)_ircUser ? _ircUser->isAway() : false;
   default:
     return PropertyMapItem::data(column, role);
   }
@@ -696,12 +788,6 @@ QString IrcUserItem::toolTip(int column) const {
   return QString("<p> %1 </p>").arg(toolTip.join("<br />"));
 }
 
-// void IrcUserItem::ircUserDestroyed() {
-//   parent()->removeChild(this);
-//   if(parent()->childCount() == 0)
-//     parent()->parent()->removeChild(parent());
-// }
-
 /*****************************************
  * NetworkModel
  *****************************************/
@@ -712,6 +798,12 @@ NetworkModel::NetworkModel(QObject *parent)
 	  this, SLOT(checkForNewBuffers(const QModelIndex &, int, int)));
   connect(this, SIGNAL(rowsAboutToBeRemoved(const QModelIndex &, int, int)),
 	  this, SLOT(checkForRemovedBuffers(const QModelIndex &, int, int)));
+
+  BufferSettings defaultSettings;
+  defaultSettings.notify("UserNoticesTarget", this, SLOT(messageRedirectionSettingsChanged()));
+  defaultSettings.notify("ServerNoticesTarget", this, SLOT(messageRedirectionSettingsChanged()));
+  defaultSettings.notify("ErrorMsgsTarget", this, SLOT(messageRedirectionSettingsChanged()));
+  messageRedirectionSettingsChanged();
 }
 
 QList<QVariant >NetworkModel::defaultHeader() {
@@ -724,7 +816,7 @@ bool NetworkModel::isBufferIndex(const QModelIndex &index) const {
   return index.data(NetworkModel::ItemTypeRole) == NetworkModel::BufferItemType;
 }
 
-int NetworkModel::networkRow(NetworkId networkId) {
+int NetworkModel::networkRow(NetworkId networkId) const {
   NetworkItem *netItem = 0;
   for(int i = 0; i < rootItem->childCount(); i++) {
     netItem = qobject_cast<NetworkItem *>(rootItem->child(i));
@@ -744,7 +836,7 @@ QModelIndex NetworkModel::networkIndex(NetworkId networkId) {
     return indexByItem(qobject_cast<NetworkItem *>(rootItem->child(netRow)));
 }
 
-NetworkItem *NetworkModel::findNetworkItem(NetworkId networkId) {
+NetworkItem *NetworkModel::findNetworkItem(NetworkId networkId) const {
   int netRow = networkRow(networkId);
   if(netRow == -1)
     return 0;
@@ -776,7 +868,7 @@ QModelIndex NetworkModel::bufferIndex(BufferId bufferId) {
   return indexByItem(_bufferItemCache[bufferId]);
 }
 
-BufferItem *NetworkModel::findBufferItem(BufferId bufferId) {
+BufferItem *NetworkModel::findBufferItem(BufferId bufferId) const {
   if(_bufferItemCache.contains(bufferId))
     return _bufferItemCache[bufferId];
   else
@@ -842,49 +934,6 @@ QMimeData *NetworkModel::mimeData(const QModelIndexList &indexes) const {
   return mimeData;
 }
 
-bool NetworkModel::dropMimeData(const QMimeData *data, Qt::DropAction action, int row, int column, const QModelIndex &parent) {
-  Q_UNUSED(action)
-  Q_UNUSED(row)
-  Q_UNUSED(column)
-
-  if(!mimeContainsBufferList(data))
-    return false;
-
-  // target must be a query
-  BufferInfo::Type targetType = (BufferInfo::Type)parent.data(NetworkModel::BufferTypeRole).toInt();
-  if(targetType != BufferInfo::QueryBuffer)
-    return false;
-
-  QList< QPair<NetworkId, BufferId> > bufferList = mimeDataToBufferList(data);
-
-  // exactly one buffer has to be dropped
-  if(bufferList.count() != 1)
-    return false;
-
-  NetworkId netId = bufferList.first().first;
-  BufferId bufferId = bufferList.first().second;
-
-  // no self merges (would kill us)
-  if(bufferId == parent.data(BufferIdRole).value<BufferId>())
-    return false; 
-
-  NetworkItem *netItem = findNetworkItem(netId);
-  Q_ASSERT(netItem);
-
-  BufferItem *bufferItem = netItem->findBufferItem(bufferId);
-  Q_ASSERT(bufferItem);
-
-  // source must be a query too
-  if(bufferItem->bufferType() != BufferInfo::QueryBuffer)
-    return false;
-    
-  // TODO: warn user about buffermerge!
-  qDebug() << "merging" << bufferId << parent.data(BufferIdRole).value<BufferId>();
-  removeRow(parent.row(), parent.parent());
-  
-  return true;
-}
-
 void NetworkModel::attachNetwork(Network *net) {
   NetworkItem *netItem = networkItem(net->networkId());
   netItem->attachNetwork(net);
@@ -904,24 +953,84 @@ void NetworkModel::removeBuffer(BufferId bufferId) {
   buffItem->parent()->removeChild(buffItem);
 }
 
-MsgId NetworkModel::lastSeenMsgId(BufferId bufferId) {
+MsgId NetworkModel::lastSeenMsgId(BufferId bufferId) const {
   if(!_bufferItemCache.contains(bufferId))
     return MsgId();
 
   return _bufferItemCache[bufferId]->lastSeenMsgId();
 }
 
+MsgId NetworkModel::lastSeenMarkerMsgId(BufferId bufferId) const {
+  if(!_bufferItemCache.contains(bufferId))
+    return MsgId();
+
+  return _bufferItemCache[bufferId]->lastSeenMarkerMsgId();
+}
+
+MsgId NetworkModel::lastSeenMsgId(const BufferId &bufferId) {
+  BufferItem *bufferItem = findBufferItem(bufferId);
+  if(!bufferItem) {
+    qDebug() << "NetworkModel::lastSeenMsgId(): buffer is unknown:" << bufferId;
+    Client::purgeKnownBufferIds();
+    return MsgId();
+  }
+  return bufferItem->lastSeenMsgId();
+}
+
 void NetworkModel::setLastSeenMsgId(const BufferId &bufferId, const MsgId &msgId) {
   BufferItem *bufferItem = findBufferItem(bufferId);
   if(!bufferItem) {
     qDebug() << "NetworkModel::setLastSeenMsgId(): buffer is unknown:" << bufferId;
+    Client::purgeKnownBufferIds();
     return;
   }
   bufferItem->setLastSeenMsgId(msgId);
 }
 
-void NetworkModel::updateBufferActivity(const Message &msg) {
-  bufferItem(msg.bufferInfo())->updateActivityLevel(msg);
+void NetworkModel::updateBufferActivity(Message &msg) {
+  int redirectionTarget = 0;
+  switch(msg.type()) {
+  case Message::Notice:
+    if(bufferType(msg.bufferId()) != BufferInfo::ChannelBuffer) {
+      msg.setFlags(msg.flags() | Message::Redirected);
+      if(msg.flags() & Message::ServerMsg) {
+	// server notice
+	redirectionTarget = _serverNoticesTarget;
+      } else {
+	redirectionTarget = _userNoticesTarget;
+      }
+    }
+    break;
+  case Message::Error:
+    msg.setFlags(msg.flags() | Message::Redirected);
+    redirectionTarget = _errorMsgsTarget;
+    break;
+  default:
+    break;
+  }
+
+  if(msg.flags() & Message::Redirected) {
+    if(redirectionTarget & BufferSettings::DefaultBuffer)
+      updateBufferActivity(bufferItem(msg.bufferInfo()), msg);
+
+    if(redirectionTarget & BufferSettings::StatusBuffer) {
+      const NetworkItem *netItem = findNetworkItem(msg.bufferInfo().networkId());
+      if(netItem) {
+	updateBufferActivity(netItem->statusBufferItem(), msg);
+      }
+    }
+  } else {
+    updateBufferActivity(bufferItem(msg.bufferInfo()), msg);
+  }
+}
+
+void NetworkModel::updateBufferActivity(BufferItem *bufferItem, const Message &msg) {
+  if(!bufferItem)
+    return;
+
+  bufferItem->updateActivityLevel(msg);
+  if(bufferItem->isCurrentBuffer())
+    emit setLastSeenMsg(bufferItem->bufferId(), msg.msgId());
 }
 
 void NetworkModel::setBufferActivity(const BufferId &bufferId, BufferInfo::ActivityLevel level) {
@@ -933,6 +1042,15 @@ void NetworkModel::setBufferActivity(const BufferId &bufferId, BufferInfo::Activ
   bufferItem->setActivityLevel(level);
 }
 
+void NetworkModel::clearBufferActivity(const BufferId &bufferId) {
+  BufferItem *bufferItem = findBufferItem(bufferId);
+  if(!bufferItem) {
+    qDebug() << "NetworkModel::clearBufferActivity(): buffer is unknown:" << bufferId;
+    return;
+  }
+  bufferItem->clearActivityLevel();
+}
+
 const Network *NetworkModel::networkByIndex(const QModelIndex &index) const {
   QVariant netVariant = index.data(NetworkIdRole);
   if(!netVariant.isValid())
@@ -945,7 +1063,7 @@ const Network *NetworkModel::networkByIndex(const QModelIndex &index) const {
 void NetworkModel::checkForRemovedBuffers(const QModelIndex &parent, int start, int end) {
   if(parent.data(ItemTypeRole) != NetworkItemType)
     return;
-  
+
   for(int row = start; row <= end; row++) {
     _bufferItemCache.remove(parent.child(row, 0).data(BufferIdRole).value<BufferId>());
   }
@@ -961,28 +1079,28 @@ void NetworkModel::checkForNewBuffers(const QModelIndex &parent, int start, int
   }
 }
 
-QString NetworkModel::bufferName(BufferId bufferId) {
+QString NetworkModel::bufferName(BufferId bufferId) const {
   if(!_bufferItemCache.contains(bufferId))
     return QString();
 
   return _bufferItemCache[bufferId]->bufferName();
 }
 
-BufferInfo::Type NetworkModel::bufferType(BufferId bufferId) {
+BufferInfo::Type NetworkModel::bufferType(BufferId bufferId) const {
   if(!_bufferItemCache.contains(bufferId))
     return BufferInfo::InvalidBuffer;
 
   return _bufferItemCache[bufferId]->bufferType();
 }
 
-BufferInfo NetworkModel::bufferInfo(BufferId bufferId) {
+BufferInfo NetworkModel::bufferInfo(BufferId bufferId) const {
   if(!_bufferItemCache.contains(bufferId))
     return BufferInfo();
 
   return _bufferItemCache[bufferId]->bufferInfo();
 }
 
-NetworkId NetworkModel::networkId(BufferId bufferId) {
+NetworkId NetworkModel::networkId(BufferId bufferId) const {
   if(!_bufferItemCache.contains(bufferId))
     return NetworkId();
 
@@ -993,7 +1111,7 @@ NetworkId NetworkModel::networkId(BufferId bufferId) {
     return NetworkId();
 }
 
-QString NetworkModel::networkName(BufferId bufferId) {
+QString NetworkModel::networkName(BufferId bufferId) const {
   if(!_bufferItemCache.contains(bufferId))
     return QString();
 
@@ -1003,3 +1121,55 @@ QString NetworkModel::networkName(BufferId bufferId) {
   else
     return QString();
 }
+
+BufferId NetworkModel::bufferId(NetworkId networkId, const QString &bufferName, Qt::CaseSensitivity cs) const {
+  const NetworkItem *netItem = findNetworkItem(networkId);
+  if(!netItem)
+    return BufferId();
+
+  for(int i = 0; i < netItem->childCount(); i++) {
+    BufferItem *bufferItem = qobject_cast<BufferItem *>(netItem->child(i));
+    if(bufferItem && !bufferItem->bufferName().compare(bufferName, cs))
+      return bufferItem->bufferId();
+  }
+  return BufferId();
+}
+
+void NetworkModel::sortBufferIds(QList<BufferId> &bufferIds) const {
+  QList<BufferItem *> bufferItems;
+  foreach(BufferId bufferId, bufferIds) {
+    if(_bufferItemCache.contains(bufferId))
+      bufferItems << _bufferItemCache[bufferId];
+  }
+
+  qSort(bufferItems.begin(), bufferItems.end(), bufferItemLessThan);
+
+  bufferIds.clear();
+  foreach(BufferItem *bufferItem, bufferItems) {
+    bufferIds << bufferItem->bufferId();
+  }
+}
+
+QList<BufferId> NetworkModel::allBufferIdsSorted() const {
+  QList<BufferId> bufferIds = allBufferIds();
+  sortBufferIds(bufferIds);
+  return bufferIds;
+}
+
+bool NetworkModel::bufferItemLessThan(const BufferItem *left, const BufferItem *right) {
+  int leftType = left->bufferType();
+  int rightType = right->bufferType();
+
+  if(leftType != rightType)
+    return leftType < rightType;
+  else
+    return QString::compare(left->bufferName(), right->bufferName(), Qt::CaseInsensitive) < 0;
+}
+
+void NetworkModel::messageRedirectionSettingsChanged() {
+  BufferSettings bufferSettings;
+
+  _userNoticesTarget = bufferSettings.userNoticesTarget();
+  _serverNoticesTarget = bufferSettings.serverNoticesTarget();
+  _errorMsgsTarget = bufferSettings.errorMsgsTarget();
+}
