@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-08 by the Quassel Project                          *
+ *   Copyright (C) 2005-09 by the Quassel Project                          *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -17,16 +17,17 @@
  *   Free Software Foundation, Inc.,                                       *
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
-#include "network.h"
-
-#include <QDebug>
+#include <QSettings>
 #include <QTextCodec>
 
-#include "util.h"
+#include "network.h"
+#include "quassel.h"
 
 QTextCodec *Network::_defaultCodecForServer = 0;
 QTextCodec *Network::_defaultCodecForEncoding = 0;
 QTextCodec *Network::_defaultCodecForDecoding = 0;
+QString Network::_networksIniPath = QString();
+
 // ====================
 //  Public:
 // ====================
@@ -51,7 +52,8 @@ Network::Network(const NetworkId &networkid, QObject *parent)
     _unlimitedReconnectRetries(false),
     _codecForServer(0),
     _codecForEncoding(0),
-    _codecForDecoding(0)
+    _codecForDecoding(0),
+    _autoAwayActive(false)
 {
   setObjectName(QString::number(networkid.toInt()));
 }
@@ -63,7 +65,7 @@ Network::~Network() {
 bool Network::isChannelName(const QString &channelname) const {
   if(channelname.isEmpty())
     return false;
-  
+
   if(supports("CHANTYPES"))
     return support("CHANTYPES").contains(channelname[0]);
   else
@@ -99,7 +101,7 @@ void Network::setNetworkInfo(const NetworkInfo &info) {
   if(info.codecForServer != codecForServer()) setCodecForServer(QTextCodec::codecForName(info.codecForServer));
   if(info.codecForEncoding != codecForEncoding()) setCodecForEncoding(QTextCodec::codecForName(info.codecForEncoding));
   if(info.codecForDecoding != codecForDecoding()) setCodecForDecoding(QTextCodec::codecForName(info.codecForDecoding));
-  if(info.serverList.count()) setServerList(info.serverList); // FIXME compare components
+  if(info.serverList.count()) setServerList(toVariantList(info.serverList)); // FIXME compare components
   if(info.useRandomServer != useRandomServer()) setUseRandomServer(info.useRandomServer);
   if(info.perform != perform()) setPerform(info.perform);
   if(info.useAutoIdentify != useAutoIdentify()) setUseAutoIdentify(info.useAutoIdentify);
@@ -139,7 +141,7 @@ QStringList Network::nicks() const {
 QString Network::prefixes() {
   if(_prefixes.isNull())
     determinePrefixes();
-  
+
   return _prefixes;
 }
 
@@ -150,7 +152,7 @@ QString Network::prefixModes() {
   return _prefixModes;
 }
 
-// example Unreal IRCD: CHANMODES=beI,kfL,lj,psmntirRcOAQKVCuzNSMTG 
+// example Unreal IRCD: CHANMODES=beI,kfL,lj,psmntirRcOAQKVCuzNSMTG
 Network::ChannelModeType Network::channelModeType(const QString &mode) {
   if(mode.isEmpty())
     return NOT_A_CHANMODE;
@@ -181,70 +183,31 @@ QString Network::support(const QString &param) const {
     return QString();
 }
 
-IrcUser *Network::newIrcUser(const QString &hostmask) {
+IrcUser *Network::newIrcUser(const QString &hostmask, const QVariantMap &initData) {
   QString nick(nickFromMask(hostmask).toLower());
   if(!_ircUsers.contains(nick)) {
     IrcUser *ircuser = new IrcUser(hostmask, this);
+    if(!initData.isEmpty()) {
+      ircuser->fromVariantMap(initData);
+      ircuser->setInitialized();
+    }
 
     if(proxy())
       proxy()->synchronize(ircuser);
     else
       qWarning() << "unable to synchronize new IrcUser" << hostmask << "forgot to call Network::setProxy(SignalProxy *)?";
-    
+
     connect(ircuser, SIGNAL(nickSet(QString)), this, SLOT(ircUserNickChanged(QString)));
-    connect(ircuser, SIGNAL(initDone()), this, SLOT(ircUserInitDone()));
-    connect(ircuser, SIGNAL(destroyed()), this, SLOT(ircUserDestroyed()));
+
     _ircUsers[nick] = ircuser;
+
     emit ircUserAdded(hostmask);
     emit ircUserAdded(ircuser);
   }
+
   return _ircUsers[nick];
 }
 
-void Network::ircUserDestroyed() {
-  IrcUser *ircUser = static_cast<IrcUser *>(sender());
-  if(!ircUser)
-    return;
-
-  QHash<QString, IrcUser *>::iterator ircUserIter = _ircUsers.begin();
-  while(ircUserIter != _ircUsers.end()) {
-    if(ircUser == *ircUserIter) {
-      ircUserIter = _ircUsers.erase(ircUserIter);
-      break;
-    }
-    ircUserIter++;
-  }
-}
-
-void Network::removeIrcUser(IrcUser *ircuser) {
-  QString nick = _ircUsers.key(ircuser);
-  if(nick.isNull())
-    return;
-
-  _ircUsers.remove(nick);
-  disconnect(ircuser, 0, this, 0);
-  emit ircUserRemoved(nick);
-  emit ircUserRemoved(ircuser);
-  ircuser->deleteLater();
-}
-
-void Network::removeIrcUser(const QString &nick) {
-  IrcUser *ircuser;
-  if((ircuser = ircUser(nick)) != 0)
-    removeIrcUser(ircuser);
-}
-
-void Network::removeChansAndUsers() {
-  QList<IrcUser *> users = ircUsers();
-  foreach(IrcUser *user, users) {
-    removeIrcUser(user);
-  }
-  QList<IrcChannel *> channels = ircChannels();
-  foreach(IrcChannel *channel, channels) {
-    removeIrcChannel(channel);
-  }
-}
-
 IrcUser *Network::ircUser(QString nickname) const {
   nickname = nickname.toLower();
   if(_ircUsers.contains(nickname))
@@ -253,18 +216,65 @@ IrcUser *Network::ircUser(QString nickname) const {
     return 0;
 }
 
-IrcChannel *Network::newIrcChannel(const QString &channelname) {
+void Network::removeIrcUser(IrcUser *ircuser) {
+  QString nick = _ircUsers.key(ircuser);
+  if(nick.isNull())
+    return;
+
+  _ircUsers.remove(nick);
+  disconnect(ircuser, 0, this, 0);
+  ircuser->deleteLater();
+}
+
+void Network::removeIrcChannel(IrcChannel *channel) {
+  QString chanName = _ircChannels.key(channel);
+  if(chanName.isNull())
+    return;
+
+  _ircChannels.remove(chanName);
+  disconnect(channel, 0, this, 0);
+  channel->deleteLater();
+}
+
+void Network::removeChansAndUsers() {
+  QList<IrcUser *> users = ircUsers();
+  _ircUsers.clear();
+  QList<IrcChannel *> channels = ircChannels();
+  _ircChannels.clear();
+
+  foreach(IrcChannel *channel, channels) {
+    proxy()->detachObject(channel);
+    disconnect(channel, 0, this, 0);
+  }
+  foreach(IrcUser *user, users) {
+    proxy()->detachObject(user);
+    disconnect(user, 0, this, 0);
+  }
+
+  // the second loop is needed because quit can have sideffects
+  foreach(IrcUser *user, users) {
+    user->quit();
+  }
+
+  qDeleteAll(users);
+  qDeleteAll(channels);
+}
+
+IrcChannel *Network::newIrcChannel(const QString &channelname, const QVariantMap &initData) {
   if(!_ircChannels.contains(channelname.toLower())) {
-    IrcChannel *channel = new IrcChannel(channelname, this);
+    IrcChannel *channel = ircChannelFactory(channelname);
+    if(!initData.isEmpty()) {
+      channel->fromVariantMap(initData);
+      channel->setInitialized();
+    }
 
     if(proxy())
       proxy()->synchronize(channel);
     else
       qWarning() << "unable to synchronize new IrcChannel" << channelname << "forgot to call Network::setProxy(SignalProxy *)?";
 
-    connect(channel, SIGNAL(initDone()), this, SLOT(ircChannelInitDone()));
-    connect(channel, SIGNAL(destroyed()), this, SLOT(channelDestroyed()));
     _ircChannels[channelname.toLower()] = channel;
+
     emit ircChannelAdded(channelname);
     emit ircChannelAdded(channel);
   }
@@ -388,6 +398,68 @@ QByteArray Network::encodeServerString(const QString &string) const {
   return string.toAscii();
 }
 
+/*** Handle networks.ini ***/
+
+QStringList Network::presetNetworks(bool onlyDefault) {
+  // lazily find the file, make sure to not call one of the other preset functions first (they'll fail else)
+  if(_networksIniPath.isNull()) {
+    _networksIniPath = Quassel::findDataFilePath("networks.ini");
+    if(_networksIniPath.isNull()) {
+      _networksIniPath = ""; // now we won't check again, as it's not null anymore
+      return QStringList();
+    }
+  }
+  if(!_networksIniPath.isEmpty()) {
+    QSettings s(_networksIniPath, QSettings::IniFormat);
+    QStringList networks = s.childGroups();
+    if(!networks.isEmpty()) {
+      // we sort the list case-insensitive
+      QMap<QString, QString> sorted;
+      foreach(QString net, networks) {
+        if(onlyDefault && !s.value(QString("%1/Default").arg(net)).toBool())
+          continue;
+        sorted[net.toLower()] = net;
+      }
+      return sorted.values();
+    }
+  }
+  return QStringList();
+}
+
+QStringList Network::presetDefaultChannels(const QString &networkName) {
+  if(_networksIniPath.isEmpty())  // be sure to have called presetNetworks() first, else this always fails
+    return QStringList();
+  QSettings s(_networksIniPath, QSettings::IniFormat);
+  return s.value(QString("%1/DefaultChannels").arg(networkName)).toStringList();
+}
+
+NetworkInfo Network::networkInfoFromPreset(const QString &networkName) {
+  NetworkInfo info;
+  if(!_networksIniPath.isEmpty()) {
+    info.networkName = networkName;
+    QSettings s(_networksIniPath, QSettings::IniFormat);
+    s.beginGroup(info.networkName);
+    foreach(QString server, s.value("Servers").toStringList()) {
+      bool ssl = false;
+      QStringList splitserver = server.split(':', QString::SkipEmptyParts);
+      if(splitserver.count() != 2) {
+        qWarning() << "Invalid server entry in networks.conf:" << server;
+        continue;
+      }
+      if(splitserver[1].at(0) == '+')
+        ssl = true;
+      uint port = splitserver[1].toUInt();
+      if(!port) {
+        qWarning() << "Invalid port entry in networks.conf:" << server;
+        continue;
+      }
+      info.serverList << Network::Server(splitserver[0].trimmed(), port, QString(), ssl);
+    }
+  }
+  return info;
+}
+
+
 // ====================
 //  Public Slots:
 // ====================
@@ -404,7 +476,7 @@ void Network::setCurrentServer(const QString &currentServer) {
 void Network::setConnected(bool connected) {
   if(_connected == connected)
     return;
-  
+
   _connected = connected;
   if(!connected) {
     setMyNick(QString());
@@ -443,7 +515,7 @@ void Network::setIdentity(IdentityId id) {
 }
 
 void Network::setServerList(const QVariantList &serverList) {
-  _serverList = serverList;
+  _serverList = fromVariantList<Server>(serverList);
   emit serverListSet(serverList);
 }
 
@@ -551,56 +623,22 @@ void Network::initSetIrcUsersAndChannels(const QVariantMap &usersAndChannels) {
     qWarning() << "Network" << networkId() << "received init data for users and channels allthough there allready are known users or channels!";
     return;
   }
-    
-  QVariantMap users = usersAndChannels.value("users").toMap();
 
+  QVariantMap users = usersAndChannels.value("users").toMap();
   QVariantMap::const_iterator userIter = users.constBegin();
   QVariantMap::const_iterator userIterEnd = users.constEnd();
-  IrcUser *ircUser = 0;
-  QString hostmask;
   while(userIter != userIterEnd) {
-    hostmask = userIter.key();
-    ircUser = new IrcUser(hostmask, this);
-    ircUser->fromVariantMap(userIter.value().toMap());
-    ircUser->setInitialized();
-    proxy()->synchronize(ircUser);
-
-    connect(ircUser, SIGNAL(nickSet(QString)), this, SLOT(ircUserNickChanged(QString)));
-    connect(ircUser, SIGNAL(destroyed()), this, SLOT(ircUserDestroyed()));
-
-    _ircUsers[nickFromMask(hostmask).toLower()] = ircUser;
-
-    emit ircUserAdded(hostmask);
-    emit ircUserAdded(ircUser);
-    emit ircUserInitDone(ircUser);
-
+    newIrcUser(userIter.key(), userIter.value().toMap());
     userIter++;
   }
 
-
   QVariantMap channels = usersAndChannels.value("channels").toMap();
   QVariantMap::const_iterator channelIter = channels.constBegin();
   QVariantMap::const_iterator channelIterEnd = channels.constEnd();
-  IrcChannel *ircChannel = 0;
-  QString channelName;
-
   while(channelIter != channelIterEnd) {
-    channelName = channelIter.key();
-    ircChannel = new IrcChannel(channelName, this);
-    ircChannel->fromVariantMap(channelIter.value().toMap());
-    ircChannel->setInitialized();
-    proxy()->synchronize(ircChannel);
-      
-    connect(ircChannel, SIGNAL(destroyed()), this, SLOT(channelDestroyed()));
-    _ircChannels[channelName.toLower()] = ircChannel;
-
-    emit ircChannelAdded(channelName);
-    emit ircChannelAdded(ircChannel);
-    emit ircChannelInitDone(ircChannel);
-
+    newIrcChannel(channelIter.key(), channelIter.value().toMap());
     channelIter++;
   }
-  
 }
 
 void Network::initSetSupports(const QVariantMap &supports) {
@@ -614,7 +652,7 @@ void Network::initSetSupports(const QVariantMap &supports) {
 IrcUser *Network::updateNickFromMask(const QString &mask) {
   QString nick(nickFromMask(mask).toLower());
   IrcUser *ircuser;
-  
+
   if(_ircUsers.contains(nick)) {
     ircuser = _ircUsers[nick];
     ircuser->updateHostmask(mask);
@@ -636,45 +674,6 @@ void Network::ircUserNickChanged(QString newnick) {
     setMyNick(newnick);
 }
 
-void Network::ircUserInitDone() {
-  IrcUser *ircuser = static_cast<IrcUser *>(sender());
-  Q_ASSERT(ircuser);
-  connect(ircuser, SIGNAL(initDone()), this, SLOT(ircUserInitDone()));
-  emit ircUserInitDone(ircuser);
-}
-
-void Network::ircChannelInitDone() {
-  IrcChannel *ircChannel = static_cast<IrcChannel *>(sender());
-  Q_ASSERT(ircChannel);
-  disconnect(ircChannel, SIGNAL(initDone()), this, SLOT(ircChannelInitDone()));
-  emit ircChannelInitDone(ircChannel);
-}
-
-void Network::removeIrcChannel(IrcChannel *channel) {
-  QString chanName = _ircChannels.key(channel);
-  if(chanName.isNull())
-    return;
-  
-  _ircChannels.remove(chanName);
-  disconnect(channel, 0, this, 0);
-  emit ircChannelRemoved(chanName);
-  emit ircChannelRemoved(channel);
-  channel->deleteLater();
-}
-
-void Network::removeIrcChannel(const QString &channel) {
-  IrcChannel *chan;
-  if((chan = ircChannel(channel)) != 0)
-    removeIrcChannel(chan);
-}
-
-void Network::channelDestroyed() {
-  IrcChannel *channel = static_cast<IrcChannel *>(sender());
-  Q_ASSERT(channel);
-  _ircChannels.remove(_ircChannels.key(channel));
-  emit ircChannelRemoved(channel);
-}
-
 void Network::emitConnectionError(const QString &errorMsg) {
   emit connectionError(errorMsg);
 }
@@ -684,16 +683,16 @@ void Network::emitConnectionError(const QString &errorMsg) {
 // ====================
 void Network::determinePrefixes() {
   // seems like we have to construct them first
-  QString PREFIX = support("PREFIX");
-  
-  if(PREFIX.startsWith("(") && PREFIX.contains(")")) {
-    _prefixes = PREFIX.section(")", 1);
-    _prefixModes = PREFIX.mid(1).section(")", 0, 0);
+  QString prefix = support("PREFIX");
+
+  if(prefix.startsWith("(") && prefix.contains(")")) {
+    _prefixes = prefix.section(")", 1);
+    _prefixModes = prefix.mid(1).section(")", 0, 0);
   } else {
     QString defaultPrefixes("~&@%+");
     QString defaultPrefixModes("qaohv");
 
-    if(PREFIX.isEmpty()) {
+    if(prefix.isEmpty()) {
       _prefixes = defaultPrefixes;
       _prefixModes = defaultPrefixModes;
       return;
@@ -701,7 +700,7 @@ void Network::determinePrefixes() {
 
     // we just assume that in PREFIX are only prefix chars stored
     for(int i = 0; i < defaultPrefixes.size(); i++) {
-      if(PREFIX.contains(defaultPrefixes[i])) {
+      if(prefix.contains(defaultPrefixes[i])) {
 	_prefixes += defaultPrefixes[i];
 	_prefixModes += defaultPrefixModes[i];
       }
@@ -709,11 +708,11 @@ void Network::determinePrefixes() {
     // check for success
     if(!_prefixes.isNull())
       return;
-    
+
     // well... our assumption was obviously wrong...
     // check if it's only prefix modes
     for(int i = 0; i < defaultPrefixes.size(); i++) {
-      if(PREFIX.contains(defaultPrefixModes[i])) {
+      if(prefix.contains(defaultPrefixModes[i])) {
 	_prefixes += defaultPrefixes[i];
 	_prefixModes += defaultPrefixModes[i];
       }
@@ -726,6 +725,21 @@ void Network::determinePrefixes() {
  * NetworkInfo
  ************************************************************************/
 
+NetworkInfo::NetworkInfo()
+: networkId(0),
+  identity(1),
+  useRandomServer(false),
+  useAutoIdentify(false),
+  autoIdentifyService("NickServ"),
+  useAutoReconnect(true),
+  autoReconnectInterval(60),
+  autoReconnectRetries(20),
+  unlimitedReconnectRetries(false),
+  rejoinChannels(true)
+{
+
+}
+
 bool NetworkInfo::operator==(const NetworkInfo &other) const {
   if(networkId != other.networkId) return false;
   if(networkName != other.networkName) return false;
@@ -759,7 +773,7 @@ QDataStream &operator<<(QDataStream &out, const NetworkInfo &info) {
   i["CodecForServer"] = info.codecForServer;
   i["CodecForEncoding"] = info.codecForEncoding;
   i["CodecForDecoding"] = info.codecForDecoding;
-  i["ServerList"] = info.serverList;
+  i["ServerList"] = toVariantList(info.serverList);
   i["UseRandomServer"] = info.useRandomServer;
   i["Perform"] = info.perform;
   i["UseAutoIdentify"] = info.useAutoIdentify;
@@ -783,7 +797,7 @@ QDataStream &operator>>(QDataStream &in, NetworkInfo &info) {
   info.codecForServer = i["CodecForServer"].toByteArray();
   info.codecForEncoding = i["CodecForEncoding"].toByteArray();
   info.codecForDecoding = i["CodecForDecoding"].toByteArray();
-  info.serverList = i["ServerList"].toList();
+  info.serverList = fromVariantList<Network::Server>(i["ServerList"].toList());
   info.useRandomServer = i["UseRandomServer"].toBool();
   info.perform = i["Perform"].toStringList();
   info.useAutoIdentify = i["UseAutoIdentify"].toBool();
@@ -799,11 +813,70 @@ QDataStream &operator>>(QDataStream &in, NetworkInfo &info) {
 
 QDebug operator<<(QDebug dbg, const NetworkInfo &i) {
   dbg.nospace() << "(id = " << i.networkId << " name = " << i.networkName << " identity = " << i.identity
-      << " codecForServer = " << i.codecForServer << " codecForEncoding = " << i.codecForEncoding << " codecForDecoding = " << i.codecForDecoding
-      << " serverList = " << i.serverList << " useRandomServer = " << i.useRandomServer << " perform = " << i.perform
-      << " useAutoIdentify = " << i.useAutoIdentify << " autoIdentifyService = " << i.autoIdentifyService << " autoIdentifyPassword = " << i.autoIdentifyPassword
-      << " useAutoReconnect = " << i.useAutoReconnect << " autoReconnectInterval = " << i.autoReconnectInterval
-      << " autoReconnectRetries = " << i.autoReconnectRetries << " unlimitedReconnectRetries = " << i.unlimitedReconnectRetries
-      << " rejoinChannels = " << i.rejoinChannels << ")";
+		<< " codecForServer = " << i.codecForServer << " codecForEncoding = " << i.codecForEncoding << " codecForDecoding = " << i.codecForDecoding
+		<< " serverList = " << i.serverList << " useRandomServer = " << i.useRandomServer << " perform = " << i.perform
+		<< " useAutoIdentify = " << i.useAutoIdentify << " autoIdentifyService = " << i.autoIdentifyService << " autoIdentifyPassword = " << i.autoIdentifyPassword
+		<< " useAutoReconnect = " << i.useAutoReconnect << " autoReconnectInterval = " << i.autoReconnectInterval
+		<< " autoReconnectRetries = " << i.autoReconnectRetries << " unlimitedReconnectRetries = " << i.unlimitedReconnectRetries
+		<< " rejoinChannels = " << i.rejoinChannels << ")";
+  return dbg.space();
+}
+
+QDataStream &operator<<(QDataStream &out, const Network::Server &server) {
+  QVariantMap serverMap;
+  serverMap["Host"] = server.host;
+  serverMap["Port"] = server.port;
+  serverMap["Password"] = server.password;
+  serverMap["UseSSL"] = server.useSsl;
+  serverMap["sslVersion"] = server.sslVersion;
+  serverMap["UseProxy"] = server.useProxy;
+  serverMap["ProxyType"] = server.proxyType;
+  serverMap["ProxyHost"] = server.proxyHost;
+  serverMap["ProxyPort"] = server.proxyPort;
+  serverMap["ProxyUser"] = server.proxyUser;
+  serverMap["ProxyPass"] = server.proxyPass;
+  out << serverMap;
+  return out;
+}
+
+QDataStream &operator>>(QDataStream &in, Network::Server &server) {
+  QVariantMap serverMap;
+  in >> serverMap;
+  server.host = serverMap["Host"].toString();
+  server.port = serverMap["Port"].toUInt();
+  server.password = serverMap["Password"].toString();
+  server.useSsl = serverMap["UseSSL"].toBool();
+  server.sslVersion = serverMap["sslVersion"].toInt();
+  server.useProxy = serverMap["UseProxy"].toBool();
+  server.proxyType = serverMap["ProxyType"].toInt();
+  server.proxyHost = serverMap["ProxyHost"].toString();
+  server.proxyPort = serverMap["ProxyPort"].toUInt();
+  server.proxyUser = serverMap["ProxyUser"].toString();
+  server.proxyPass = serverMap["ProxyPass"].toString();
+  return in;
+}
+
+
+bool Network::Server::operator==(const Server &other) const {
+  if(host != other.host) return false;
+  if(port != other.port) return false;
+  if(password != other.password) return false;
+  if(useSsl != other.useSsl) return false;
+  if(sslVersion != other.sslVersion) return false;
+  if(useProxy != other.useProxy) return false;
+  if(proxyType != other.proxyType) return false;
+  if(proxyHost != other.proxyHost) return false;
+  if(proxyPort != other.proxyPort) return false;
+  if(proxyUser != other.proxyUser) return false;
+  if(proxyPass != other.proxyPass) return false;
+  return true;
+}
+
+bool Network::Server::operator!=(const Server &other) const {
+  return !(*this == other);
+}
+
+QDebug operator<<(QDebug dbg, const Network::Server &server) {
+  dbg.nospace() << "Server(host = " << server.host << ":" << server.port << ", useSsl = " << server.useSsl << ")";
   return dbg.space();
 }
