@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-08 by the Quassel Project                          *
+ *   Copyright (C) 2005-09 by the Quassel Project                          *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -18,14 +18,12 @@
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
  ***************************************************************************/
 
-#include <QMetaObject>
-#include <QMetaMethod>
-#include <QMutexLocker>
 #include <QCoreApplication>
 
 #include "core.h"
 #include "coresession.h"
 #include "coresettings.h"
+#include "quassel.h"
 #include "signalproxy.h"
 #include "sqlitestorage.h"
 #include "network.h"
@@ -33,8 +31,10 @@
 
 #include "util.h"
 
+// migration related
+#include <QFile>
+
 Core *Core::instanceptr = 0;
-QMutex Core::mutex;
 
 Core *Core::instance() {
   if(instanceptr) return instanceptr;
@@ -48,45 +48,113 @@ void Core::destroy() {
   instanceptr = 0;
 }
 
-Core::Core() : storage(0) {
-  _startTime = QDateTime::currentDateTime();  // for uptime :)
+Core::Core()
+  : _storage(0)
+{
+  _startTime = QDateTime::currentDateTime().toUTC();  // for uptime :)
+
+  Quassel::loadTranslation(QLocale::system());
+
+  // FIXME: MIGRATION 0.3 -> 0.4: Move database and core config to new location
+  // Move settings, note this does not delete the old files
+#ifdef Q_WS_MAC
+    QSettings newSettings("quassel-irc.org", "quasselcore");
+#else
+
+# ifdef Q_WS_WIN
+    QSettings::Format format = QSettings::IniFormat;
+# else
+    QSettings::Format format = QSettings::NativeFormat;
+# endif
+    QString newFilePath = Quassel::configDirPath() + "quasselcore"
+    + ((format == QSettings::NativeFormat) ? QLatin1String(".conf") : QLatin1String(".ini"));
+    QSettings newSettings(newFilePath, format);
+#endif /* Q_WS_MAC */
+
+  if(newSettings.value("Config/Version").toUInt() == 0) {
+#   ifdef Q_WS_MAC
+    QString org = "quassel-irc.org";
+#   else
+    QString org = "Quassel Project";
+#   endif
+    QSettings oldSettings(org, "Quassel Core");
+    if(oldSettings.allKeys().count()) {
+      qWarning() << "\n\n*** IMPORTANT: Config and data file locations have changed. Attempting to auto-migrate your core settings...";
+      foreach(QString key, oldSettings.allKeys())
+        newSettings.setValue(key, oldSettings.value(key));
+      newSettings.setValue("Config/Version", 1);
+      qWarning() << "*   Your core settings have been migrated to" << newSettings.fileName();
+
+#ifndef Q_WS_MAC /* we don't need to move the db and cert for mac */
+#ifdef Q_OS_WIN32
+      QString quasselDir = qgetenv("APPDATA") + "/quassel/";
+#elif defined Q_WS_MAC
+      QString quasselDir = QDir::homePath() + "/Library/Application Support/Quassel/";
+#else
+      QString quasselDir = QDir::homePath() + "/.quassel/";
+#endif
+
+      QFileInfo info(Quassel::configDirPath() + "quassel-storage.sqlite");
+      if(!info.exists()) {
+      // move database, if we found it
+        QFile oldDb(quasselDir + "quassel-storage.sqlite");
+        if(oldDb.exists()) {
+          bool success = oldDb.rename(Quassel::configDirPath() + "quassel-storage.sqlite");
+          if(success)
+            qWarning() << "*   Your database has been moved to" << Quassel::configDirPath() + "quassel-storage.sqlite";
+          else
+            qWarning() << "!!! Moving your database has failed. Please move it manually into" << Quassel::configDirPath();
+        }
+      }
+      // move certificate
+      QFileInfo certInfo(quasselDir + "quasselCert.pem");
+      if(certInfo.exists()) {
+        QFile cert(quasselDir + "quasselCert.pem");
+        bool success = cert.rename(Quassel::configDirPath() + "quasselCert.pem");
+        if(success)
+          qWarning() << "*   Your certificate has been moved to" << Quassel::configDirPath() + "quasselCert.pem";
+        else
+          qWarning() << "!!! Moving your certificate has failed. Please move it manually into" << Quassel::configDirPath();
+      }
+#endif /* !Q_WS_MAC */
+      qWarning() << "*** Migration completed.\n\n";
+    }
+  }
+  // MIGRATION end
+
+  // check settings version
+  // so far, we only have 1
+  CoreSettings s;
+  if(s.version() != 1) {
+    qCritical() << "Invalid core settings version, terminating!";
+    exit(EXIT_FAILURE);
+  }
 
   // Register storage backends here!
   registerStorageBackend(new SqliteStorage(this));
 
-  if(!_storageBackends.count()) {
-    quWarning() << qPrintable(tr("Could not initialize any storage backend! Exiting..."));
-    quWarning() << qPrintable(tr("Currently, Quassel only supports SQLite3. You need to build your\n"
-                                "Qt library with the sqlite plugin enabled in order for quasselcore\n"
-                                "to work."));
-    exit(1); // TODO make this less brutal (especially for mono client -> popup)
-  }
   connect(&_storageSyncTimer, SIGNAL(timeout()), this, SLOT(syncStorage()));
-  _storageSyncTimer.start(10 * 60 * 1000); // in msecs
+  _storageSyncTimer.start(10 * 60 * 1000); // 10 minutes
 }
 
 void Core::init() {
-  configured = false;
-
   CoreSettings cs;
+  _configured = initStorage(cs.storageSettings().toMap());
 
-  if(!(configured = initStorage(cs.storageSettings().toMap()))) {
-    quWarning() << "Core is currently not configured! Please connect with a Quassel Client for basic setup.";
-
-    // try to migrate old settings
-    QVariantMap old = cs.oldDbSettings().toMap();
-    if(old.count() && old["Type"].toString().toUpper() == "SQLITE") {
-      QVariantMap newSettings;
-      newSettings["Backend"] = "SQLite";
-      if((configured = initStorage(newSettings))) {
-        quWarning() << "...but thankfully I found some old settings to migrate!";
-        cs.setStorageSettings(newSettings);
-      }
+  if(!_configured) {
+    if(!_storageBackends.count()) {
+      qWarning() << qPrintable(tr("Could not initialize any storage backend! Exiting..."));
+      qWarning() << qPrintable(tr("Currently, Quassel only supports SQLite3. You need to build your\n"
+				  "Qt library with the sqlite plugin enabled in order for quasselcore\n"
+				  "to work."));
+      exit(1); // TODO make this less brutal (especially for mono client -> popup)
     }
+    qWarning() << "Core is currently not configured! Please connect with a Quassel Client for basic setup.";
   }
 
-  connect(&server, SIGNAL(newConnection()), this, SLOT(incomingConnection()));
-  if(!startListening(cs.port())) exit(1); // TODO make this less brutal
+  connect(&_server, SIGNAL(newConnection()), this, SLOT(incomingConnection()));
+  connect(&_v6server, SIGNAL(newConnection()), this, SLOT(incomingConnection()));
+  if(!startListening()) exit(1); // TODO make this less brutal
 }
 
 Core::~Core() {
@@ -110,12 +178,12 @@ void Core::saveState() {
 }
 
 void Core::restoreState() {
-  if(!instance()->configured) {
+  if(!instance()->_configured) {
     // qWarning() << qPrintable(tr("Cannot restore a state for an unconfigured core!"));
     return;
   }
   if(instance()->sessions.count()) {
-    quWarning() << qPrintable(tr("Calling restoreState() even though active sessions exist!"));
+    qWarning() << qPrintable(tr("Calling restoreState() even though active sessions exist!"));
     return;
   }
   CoreSettings s;
@@ -137,23 +205,35 @@ void Core::restoreState() {
 }
 
 /*** Core Setup ***/
+QString Core::setupCoreForInternalUsage() {
+  Q_ASSERT(!_storageBackends.isEmpty());
+  QVariantMap setupData;
+  qsrand(QDateTime::currentDateTime().toTime_t());
+  int pass = 0;
+  for(int i = 0; i < 10; i++) {
+    pass *= 10;
+    pass += qrand() % 10;
+  }
+  setupData["AdminUser"] = "AdminUser";
+  setupData["AdminPasswd"] = QString::number(pass);
+  setupData["Backend"] = _storageBackends[_storageBackends.keys().first()]->displayName();
+  return setupCore(setupData);
+}
 
-QString Core::setupCore(const QVariant &setupData_) {
-  QVariantMap setupData = setupData_.toMap();
+QString Core::setupCore(QVariantMap setupData) {
   QString user = setupData.take("AdminUser").toString();
   QString password = setupData.take("AdminPasswd").toString();
   if(user.isEmpty() || password.isEmpty()) {
     return tr("Admin user or password not set.");
   }
-  if(!initStorage(setupData, true)) {
+  _configured = initStorage(setupData, true);
+  if(!_configured) {
     return tr("Could not setup storage!");
   }
   CoreSettings s;
   s.setStorageSettings(setupData);
   quInfo() << qPrintable(tr("Creating admin user..."));
-  mutex.lock();
-  storage->addUser(user, password);
-  mutex.unlock();
+  _storage->addUser(user, password);
   startListening();  // TODO check when we need this
   return QString();
 }
@@ -178,54 +258,55 @@ void Core::unregisterStorageBackend(Storage *backend) {
 // old db settings:
 // "Type" => "sqlite"
 bool Core::initStorage(QVariantMap dbSettings, bool setup) {
+  _storage = 0;
+
   QString backend = dbSettings["Backend"].toString();
   if(backend.isEmpty()) {
-    //qWarning() << "No storage backend selected!";
-    return configured = false;
+    return false;
   }
 
+  Storage *storage = 0;
   if(_storageBackends.contains(backend)) {
     storage = _storageBackends[backend];
   } else {
-    quError() << "Selected storage backend is not available:" << backend;
-    return configured = false;
+    qCritical() << "Selected storage backend is not available:" << backend;
+    return false;
   }
-  if(!storage->init(dbSettings)) {
-    if(!setup || !(storage->setup(dbSettings) && storage->init(dbSettings))) {
-      quError() << "Could not init storage!";
-      storage = 0;
-      return configured = false;
-    }
-  }
-  // delete all other backends
-  foreach(Storage *s, _storageBackends.values()) {
-    if(s != storage) s->deleteLater();
-  }
-  _storageBackends.clear();
 
-  connect(storage, SIGNAL(bufferInfoUpdated(UserId, const BufferInfo &)), this, SIGNAL(bufferInfoUpdated(UserId, const BufferInfo &)));
-  return configured = true;
+  Storage::State storageState = storage->init(dbSettings);
+  switch(storageState) {
+  case Storage::NeedsSetup:
+    if(!setup)
+      return false; // trigger setup process
+    if(storage->setup(dbSettings))
+      return initStorage(dbSettings, false);
+    // if setup wasn't successfull we mark the backend as unavailable
+  case Storage::NotAvailable:
+    qCritical() << "Selected storage backend is not available:" << backend;
+    storage->deleteLater();
+    _storageBackends.remove(backend);
+    storage = 0;
+    return false;
+  case Storage::IsReady:
+    // delete all other backends
+    foreach(Storage *s, _storageBackends.values()) {
+      if(s != storage) s->deleteLater();
+    }
+    _storageBackends.clear();
+    connect(storage, SIGNAL(bufferInfoUpdated(UserId, const BufferInfo &)), this, SIGNAL(bufferInfoUpdated(UserId, const BufferInfo &)));
+  }
+  _storage = storage;
+  return true;
 }
 
 void Core::syncStorage() {
-  QMutexLocker locker(&mutex);
-  if(storage) storage->sync();
+  if(_storage)
+    _storage->sync();
 }
 
 /*** Storage Access ***/
-void Core::setUserSetting(UserId userId, const QString &settingName, const QVariant &data) {
-  QMutexLocker locker(&mutex);
-  instance()->storage->setUserSetting(userId, settingName, data);
-}
-
-QVariant Core::getUserSetting(UserId userId, const QString &settingName, const QVariant &data) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->getUserSetting(userId, settingName, data);
-}
-
 bool Core::createNetwork(UserId user, NetworkInfo &info) {
-  QMutexLocker locker(&mutex);
-  NetworkId networkId = instance()->storage->createNetwork(user, info);
+  NetworkId networkId = instance()->_storage->createNetwork(user, info);
   if(!networkId.isValid())
     return false;
 
@@ -233,152 +314,113 @@ bool Core::createNetwork(UserId user, NetworkInfo &info) {
   return true;
 }
 
-bool Core::updateNetwork(UserId user, const NetworkInfo &info) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->updateNetwork(user, info);
-}
-
-bool Core::removeNetwork(UserId user, const NetworkId &networkId) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->removeNetwork(user, networkId);
-}
-
-QList<NetworkInfo> Core::networks(UserId user) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->networks(user);
-}
-
-NetworkId Core::networkId(UserId user, const QString &network) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->getNetworkId(user, network);
-}
-
-QList<NetworkId> Core::connectedNetworks(UserId user) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->connectedNetworks(user);
-}
-
-void Core::setNetworkConnected(UserId user, const NetworkId &networkId, bool isConnected) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->setNetworkConnected(user, networkId, isConnected);
-}
-
-QHash<QString, QString> Core::persistentChannels(UserId user, const NetworkId &networkId) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->persistentChannels(user, networkId);
-}
-
-void Core::setChannelPersistent(UserId user, const NetworkId &networkId, const QString &channel, bool isJoined) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->setChannelPersistent(user, networkId, channel, isJoined);
-}
-
-void Core::setPersistentChannelKey(UserId user, const NetworkId &networkId, const QString &channel, const QString &key) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->setPersistentChannelKey(user, networkId, channel, key);
-}
-
-BufferInfo Core::bufferInfo(UserId user, const NetworkId &networkId, BufferInfo::Type type, const QString &buffer) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->getBufferInfo(user, networkId, type, buffer);
-}
-
-BufferInfo Core::getBufferInfo(UserId user, const BufferId &bufferId) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->getBufferInfo(user, bufferId);
-}
-
-MsgId Core::storeMessage(const Message &message) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->logMessage(message);
-}
-
-QList<Message> Core::requestMsgs(UserId user, BufferId buffer, int lastmsgs, int offset) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->requestMsgs(user, buffer, lastmsgs, offset);
-}
-
-QList<Message> Core::requestMsgs(UserId user, BufferId buffer, QDateTime since, int offset) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->requestMsgs(user, buffer, since, offset);
-}
-
-QList<Message> Core::requestMsgRange(UserId user, BufferId buffer, int first, int last) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->requestMsgRange(user, buffer, first, last);
-}
-
-QList<BufferInfo> Core::requestBuffers(UserId user) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->requestBuffers(user);
-}
-
-QList<BufferId> Core::requestBufferIdsForNetwork(UserId user, NetworkId networkId) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->requestBufferIdsForNetwork(user, networkId);
-}
-
-bool Core::removeBuffer(const UserId &user, const BufferId &bufferId) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->removeBuffer(user, bufferId);
-}
-
-BufferId Core::renameBuffer(const UserId &user, const NetworkId &networkId, const QString &newName, const QString &oldName) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->renameBuffer(user, networkId, newName, oldName);
-}
-
-void Core::setBufferLastSeenMsg(UserId user, const BufferId &bufferId, const MsgId &msgId) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->setBufferLastSeenMsg(user, bufferId, msgId);
-}
-
-QHash<BufferId, MsgId> Core::bufferLastSeenMsgIds(UserId user) {
-  QMutexLocker locker(&mutex);
-  return instance()->storage->bufferLastSeenMsgIds(user);
-}
-
 /*** Network Management ***/
 
-bool Core::startListening(uint port) {
+bool Core::startListening() {
+  // in mono mode we only start a local port if a port is specified in the cli call
+  if(Quassel::runMode() == Quassel::Monolithic && !Quassel::isOptionSet("port"))
+    return true;
+
   bool success = false;
+  uint port = Quassel::optionValue("port").toUInt();
 
-  // let's see if ipv6 is available
-  success = server.listen(QHostAddress::AnyIPv6, port);
-
-  if(!success && server.serverError() == QAbstractSocket::UnsupportedSocketOperationError) {
-    // fall back to v4
-    success = server.listen(QHostAddress::Any, port);
+  const QString listen = Quassel::optionValue("listen");
+  const QStringList listen_list = listen.split(",", QString::SkipEmptyParts);
+  if(listen_list.size() > 0) {
+    foreach (const QString listen_term, listen_list) {  // TODO: handle multiple interfaces for same TCP version gracefully
+      QHostAddress addr;
+      if(!addr.setAddress(listen_term)) {
+        qCritical() << qPrintable(
+          tr("Invalid listen address %1")
+            .arg(listen_term)
+        );
+      } else {
+        switch(addr.protocol()) {
+          case QAbstractSocket::IPv4Protocol:
+            if(_server.listen(addr, port)) {
+              quInfo() << qPrintable(
+                tr("Listening for GUI clients on IPv4 %1 port %2 using protocol version %3")
+                  .arg(addr.toString())
+                  .arg(_server.serverPort())
+                  .arg(Quassel::buildInfo().protocolVersion)
+              );
+              success = true;
+            } else
+              quWarning() << qPrintable(
+                tr("Could not open IPv4 interface %1:%2: %3")
+                  .arg(addr.toString())
+                  .arg(port)
+                  .arg(_server.errorString()));
+            break;
+          case QAbstractSocket::IPv6Protocol:
+            if(_v6server.listen(addr, port)) {
+              quInfo() << qPrintable(
+                tr("Listening for GUI clients on IPv6 %1 port %2 using protocol version %3")
+                  .arg(addr.toString())
+                  .arg(_v6server.serverPort())
+                  .arg(Quassel::buildInfo().protocolVersion)
+              );
+              success = true;
+            } else {
+              // if v4 succeeded on Any, the port will be already in use - don't display the error then
+              // FIXME: handle this more sanely, make sure we can listen to both v4 and v6 by default!
+              if(!success || _v6server.serverError() != QAbstractSocket::AddressInUseError)
+                quWarning() << qPrintable(
+                  tr("Could not open IPv6 interface %1:%2: %3")
+                  .arg(addr.toString())
+                  .arg(port)
+                  .arg(_v6server.errorString()));
+            }
+            break;
+          default:
+            qCritical() << qPrintable(
+              tr("Invalid listen address %1, unknown network protocol")
+                  .arg(listen_term)
+            );
+            break;
+        }
+      }
+    }
   }
+  if(!success)
+    quError() << qPrintable(tr("Could not open any network interfaces to listen on!"));
 
-  if(!success) {
-    quError() << qPrintable(QString("Could not open GUI client port %1: %2").arg(port).arg(server.errorString()));
-  } else {
-    quInfo() << "Listening for GUI clients on port" << server.serverPort() << "using protocol version" << Global::protocolVersion;
-  }
-  
   return success;
 }
 
-void Core::stopListening() {
-  server.close();
-  quInfo() << "No longer listening for GUI clients.";
+void Core::stopListening(const QString &reason) {
+  bool wasListening = false;
+  if(_server.isListening()) {
+    wasListening = true;
+    _server.close();
+  }
+  if(_v6server.isListening()) {
+    wasListening = true;
+    _v6server.close();
+  }
+  if(wasListening) {
+    if(reason.isEmpty())
+      quInfo() << "No longer listening for GUI clients.";
+    else
+      quInfo() << qPrintable(reason);
+  }
 }
 
 void Core::incomingConnection() {
-  while(server.hasPendingConnections()) {
-    QTcpSocket *socket = server.nextPendingConnection();
+  QTcpServer *server = qobject_cast<QTcpServer *>(sender());
+  Q_ASSERT(server);
+  while(server->hasPendingConnections()) {
+    QTcpSocket *socket = server->nextPendingConnection();
     connect(socket, SIGNAL(disconnected()), this, SLOT(clientDisconnected()));
     connect(socket, SIGNAL(readyRead()), this, SLOT(clientHasData()));
     connect(socket, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(socketError(QAbstractSocket::SocketError)));
-    
+
     QVariantMap clientInfo;
     blocksizes.insert(socket, (quint32)0);
     quInfo() << qPrintable(tr("Client connected from"))  << qPrintable(socket->peerAddress().toString());
 
-    if (!configured) {
-      server.close();
-      quDebug() << "Closing server for basic setup.";
+    if(!_configured) {
+      stopListening(tr("Closing server for basic setup."));
     }
   }
 }
@@ -397,7 +439,7 @@ void Core::clientHasData() {
 void Core::processClientMessage(QTcpSocket *socket, const QVariantMap &msg) {
   if(!msg.contains("MsgType")) {
     // Client is way too old, does not even use the current init format
-    quWarning() << qPrintable(tr("Antique client trying to connect... refusing."));
+    qWarning() << qPrintable(tr("Antique client trying to connect... refusing."));
     socket->close();
     return;
   }
@@ -406,37 +448,35 @@ void Core::processClientMessage(QTcpSocket *socket, const QVariantMap &msg) {
     QVariantMap reply;
 
     // Just version information -- check it!
-    uint ver = 0;
-    if(!msg.contains("ProtocolVersion") && msg["ClientBuild"].toUInt() >= 732) ver = 1; // FIXME legacy
-    if(msg.contains("ProtocolVersion")) ver = msg["ProtocolVersion"].toUInt();
-    if(ver < Global::coreNeedsProtocol) {
+    uint ver = msg["ProtocolVersion"].toUInt();
+    if(ver < Quassel::buildInfo().coreNeedsProtocol) {
       reply["MsgType"] = "ClientInitReject";
       reply["Error"] = tr("<b>Your Quassel Client is too old!</b><br>"
       "This core needs at least client/core protocol version %1.<br>"
-      "Please consider upgrading your client.").arg(Global::coreNeedsProtocol);
+      "Please consider upgrading your client.").arg(Quassel::buildInfo().coreNeedsProtocol);
       SignalProxy::writeDataToDevice(socket, reply);
-      quWarning() << qPrintable(tr("Client")) << qPrintable(socket->peerAddress().toString()) << qPrintable(tr("too old, rejecting."));
+      qWarning() << qPrintable(tr("Client")) << qPrintable(socket->peerAddress().toString()) << qPrintable(tr("too old, rejecting."));
       socket->close(); return;
     }
 
-    reply["CoreVersion"] = Global::quasselVersion;
-    reply["CoreDate"] = Global::quasselBuildDate;
-    reply["CoreBuild"] = 860; // FIXME legacy
-    reply["ProtocolVersion"] = Global::protocolVersion;
+    reply["CoreVersion"] = Quassel::buildInfo().fancyVersionString;
+    reply["CoreDate"] = Quassel::buildInfo().buildDate;
+    reply["ProtocolVersion"] = Quassel::buildInfo().protocolVersion;
     // TODO: Make the core info configurable
-    int uptime = startTime().secsTo(QDateTime::currentDateTime());
+    int uptime = startTime().secsTo(QDateTime::currentDateTime().toUTC());
     int updays = uptime / 86400; uptime %= 86400;
     int uphours = uptime / 3600; uptime %= 3600;
     int upmins = uptime / 60;
     reply["CoreInfo"] = tr("<b>Quassel Core Version %1</b><br>"
 			   "Built: %2<br>"
-			   "Up %3d%4h%5m (since %6)").arg(Global::quasselVersion).arg(Global::quasselBuildDate)
+			   "Up %3d%4h%5m (since %6)").arg(Quassel::buildInfo().fancyVersionString)
+                                                     .arg(Quassel::buildInfo().buildDate)
       .arg(updays).arg(uphours,2,10,QChar('0')).arg(upmins,2,10,QChar('0')).arg(startTime().toString(Qt::TextDate));
 
-#ifndef QT_NO_OPENSSL
-    SslServer *sslServer = qobject_cast<SslServer *>(&server);
+#ifdef HAVE_SSL
+    SslServer *sslServer = qobject_cast<SslServer *>(&_server);
     QSslSocket *sslSocket = qobject_cast<QSslSocket *>(socket);
-    bool supportSsl = (bool)sslServer && (bool)sslSocket && sslServer->certIsValid();
+    bool supportSsl = (bool)sslServer && (bool)sslSocket && sslServer->isCertValid();
 #else
     bool supportSsl = false;
 #endif
@@ -454,7 +494,7 @@ void Core::processClientMessage(QTcpSocket *socket, const QVariantMap &msg) {
     reply["LoginEnabled"] = true;
 
     // check if we are configured, start wizard otherwise
-    if(!configured) {
+    if(!_configured) {
       reply["Configured"] = false;
       QList<QVariant> backends;
       foreach(Storage *backend, _storageBackends.values()) {
@@ -472,10 +512,10 @@ void Core::processClientMessage(QTcpSocket *socket, const QVariantMap &msg) {
     reply["MsgType"] = "ClientInitAck";
     SignalProxy::writeDataToDevice(socket, reply);
 
-#ifndef QT_NO_OPENSSL
+#ifdef HAVE_SSL
     // after we told the client that we are ssl capable we switch to ssl mode
     if(supportSsl && msg["UseSsl"].toBool()) {
-      quDebug() << qPrintable(tr("Starting TLS for Client:"))  << qPrintable(socket->peerAddress().toString());
+      qDebug() << qPrintable(tr("Starting TLS for Client:"))  << qPrintable(socket->peerAddress().toString());
       connect(sslSocket, SIGNAL(sslErrors(const QList<QSslError> &)), this, SLOT(sslErrors(const QList<QSslError> &)));
       sslSocket->startServerEncryption();
     }
@@ -484,10 +524,10 @@ void Core::processClientMessage(QTcpSocket *socket, const QVariantMap &msg) {
 #ifndef QT_NO_COMPRESS
     if(supportsCompression && msg["UseCompression"].toBool()) {
       socket->setProperty("UseCompression", true);
-      quDebug() << "Using compression for Client:" << qPrintable(socket->peerAddress().toString());
+      qDebug() << "Using compression for Client:" << qPrintable(socket->peerAddress().toString());
     }
 #endif
-    
+
   } else {
     // for the rest, we need an initialized connection
     if(!clientInfo.contains(socket)) {
@@ -495,12 +535,12 @@ void Core::processClientMessage(QTcpSocket *socket, const QVariantMap &msg) {
       reply["MsgType"] = "ClientLoginReject";
       reply["Error"] = tr("<b>Client not initialized!</b><br>You need to send an init message before trying to login.");
       SignalProxy::writeDataToDevice(socket, reply);
-      quWarning() << qPrintable(tr("Client")) << qPrintable(socket->peerAddress().toString()) << qPrintable(tr("did not send an init message before trying to login, rejecting."));
+      qWarning() << qPrintable(tr("Client")) << qPrintable(socket->peerAddress().toString()) << qPrintable(tr("did not send an init message before trying to login, rejecting."));
       socket->close(); return;
     }
     if(msg["MsgType"] == "CoreSetupData") {
       QVariantMap reply;
-      QString result = setupCore(msg["SetupData"]);
+      QString result = setupCore(msg["SetupData"].toMap());
       if(!result.isEmpty()) {
         reply["MsgType"] = "CoreSetupReject";
         reply["Error"] = result;
@@ -510,9 +550,7 @@ void Core::processClientMessage(QTcpSocket *socket, const QVariantMap &msg) {
       SignalProxy::writeDataToDevice(socket, reply);
     } else if(msg["MsgType"] == "ClientLogin") {
       QVariantMap reply;
-      mutex.lock();
-      UserId uid = storage->validateUser(msg["User"].toString(), msg["Password"].toString());
-      mutex.unlock();
+      UserId uid = _storage->validateUser(msg["User"].toString(), msg["Password"].toString());
       if(uid == 0) {
         reply["MsgType"] = "ClientLoginReject";
         reply["Error"] = tr("<b>Invalid username or password!</b><br>The username/password combination you supplied could not be found in the database.");
@@ -538,31 +576,33 @@ void Core::clientDisconnected() {
     socket->deleteLater();
   } else {
     // we have to crawl through the hashes and see if we find a victim to remove
-    quDebug() << qPrintable(tr("Non-authed client disconnected. (socket allready destroyed)"));
+    qDebug() << qPrintable(tr("Non-authed client disconnected. (socket allready destroyed)"));
 
     // DO NOT CALL ANY METHODS ON socket!!
     socket = static_cast<QTcpSocket *>(sender());
-    
+
     QHash<QTcpSocket *, quint32>::iterator blockSizeIter = blocksizes.begin();
     while(blockSizeIter != blocksizes.end()) {
       if(blockSizeIter.key() == socket) {
-	blocksizes.erase(blockSizeIter);
+	blockSizeIter = blocksizes.erase(blockSizeIter);
+      } else {
+	blockSizeIter++;
       }
-      blockSizeIter++;
     }
 
     QHash<QTcpSocket *, QVariantMap>::iterator clientInfoIter = clientInfo.begin();
     while(clientInfoIter != clientInfo.end()) {
       if(clientInfoIter.key() == socket) {
-	clientInfo.erase(clientInfoIter);
+	clientInfoIter = clientInfo.erase(clientInfoIter);
+      } else {
+	clientInfoIter++;
       }
-      clientInfoIter++;
     }
   }
 
 
   // make server listen again if still not configured
-  if (!configured) {
+  if (!_configured) {
     startListening();
   }
 
@@ -580,15 +620,38 @@ void Core::setupClientSession(QTcpSocket *socket, UserId uid) {
   blocksizes.remove(socket);
   clientInfo.remove(socket);
   if(!sess) {
-    quWarning() << qPrintable(tr("Could not initialize session for client:")) << qPrintable(socket->peerAddress().toString());
+    qWarning() << qPrintable(tr("Could not initialize session for client:")) << qPrintable(socket->peerAddress().toString());
     socket->close();
   }
   sess->addClient(socket);
 }
 
+void Core::setupInternalClientSession(SignalProxy *proxy) {
+  if(!_configured) {
+    stopListening();
+    setupCoreForInternalUsage();
+  }
+
+  UserId uid;
+  if(_storage) {
+    uid = _storage->internalUser();
+  } else {
+    qWarning() << "Core::setupInternalClientSession(): You're trying to run monolithic Quassel with an unusable Backend! Go fix it!";
+    return;
+  }
+
+  // Find or create session for validated user
+  SessionThread *sess;
+  if(sessions.contains(uid))
+    sess = sessions[uid];
+  else
+    sess = createSession(uid);
+  sess->addClient(proxy);
+}
+
 SessionThread *Core::createSession(UserId uid, bool restore) {
   if(sessions.contains(uid)) {
-    quWarning() << "Calling createSession() when a session for the user already exists!";
+    qWarning() << "Calling createSession() when a session for the user already exists!";
     return 0;
   }
   SessionThread *sess = new SessionThread(uid, restore, this);
@@ -597,7 +660,7 @@ SessionThread *Core::createSession(UserId uid, bool restore) {
   return sess;
 }
 
-#ifndef QT_NO_OPENSSL
+#ifdef HAVE_SSL
 void Core::sslErrors(const QList<QSslError> &errors) {
   Q_UNUSED(errors);
   QSslSocket *socket = qobject_cast<QSslSocket *>(sender());
@@ -609,5 +672,5 @@ void Core::sslErrors(const QList<QSslError> &errors) {
 void Core::socketError(QAbstractSocket::SocketError err) {
   QAbstractSocket *socket = qobject_cast<QAbstractSocket *>(sender());
   if(socket && err != QAbstractSocket::RemoteHostClosedError)
-    quWarning() << "Core::socketError()" << socket << err << socket->errorString();
+    qWarning() << "Core::socketError()" << socket << err << socket->errorString();
 }
