@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-08 by the Quassel Project                          *
+ *   Copyright (C) 2005-09 by the Quassel Project                          *
  *   devel@quassel-irc.org                                                 *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -20,78 +20,57 @@
 
 #include "bufferview.h"
 
-#include "buffermodel.h"
-#include "bufferviewfilter.h"
-#include "buffersyncer.h"
-#include "client.h"
-#include "mappedselectionmodel.h"
-#include "network.h"
-#include "networkmodel.h"
-
-#include "uisettings.h"
-
-#include "global.h"
-
+#include <QApplication>
 #include <QAction>
 #include <QFlags>
 #include <QHeaderView>
-#include <QInputDialog>
 #include <QLineEdit>
 #include <QMenu>
 #include <QMessageBox>
 #include <QSet>
 
+#include "action.h"
+#include "buffermodel.h"
+#include "bufferviewfilter.h"
+#include "buffersettings.h"
+#include "buffersyncer.h"
+#include "client.h"
+#include "contextmenuactionprovider.h"
+#include "graphicalui.h"
+#include "iconloader.h"
+#include "network.h"
+#include "networkmodel.h"
+#include "contextmenuactionprovider.h"
+#include "uisettings.h"
+
 /*****************************************
 * The TreeView showing the Buffers
 *****************************************/
 // Please be carefull when reimplementing methods which are used to inform the view about changes to the data
 // to be on the safe side: call QTreeView's method aswell
 BufferView::BufferView(QWidget *parent)
-  : QTreeView(parent),
-    showChannelList(tr("Show Channel List"), this),
-    _connectNetAction(tr("Connect"), this),
-    _disconnectNetAction(tr("Disconnect"), this),
-    _joinChannelAction(tr("Join Channel"), this),
-
-    _joinBufferAction(tr("Join"), this),
-    _partBufferAction(tr("Part"), this),
-    _hideBufferTemporarilyAction(tr("Hide buffers"), this),
-    _hideBufferPermanentlyAction(tr("Hide buffers permanently"), this),
-    _removeBufferAction(tr("Delete buffer"), this),
-    _ignoreListAction(tr("Ignore list"), this),
-
-    _hideJoinAction(tr("Join Events"), this),
-    _hidePartAction(tr("Part Events"), this),
-    _hideKillAction(tr("Kill Events"), this),
-    _hideQuitAction(tr("Quit Events"), this),
-    _hideModeAction(tr("Mode Events"), this)
-
+  : QTreeView(parent)
 {
-  _hideJoinAction.setCheckable(true);
-  _hidePartAction.setCheckable(true);
-  _hideKillAction.setCheckable(true);
-  _hideQuitAction.setCheckable(true);
-  _hideModeAction.setCheckable(true);
-  _hideJoinAction.setEnabled(false);
-  _hidePartAction.setEnabled(false);
-  _ignoreListAction.setEnabled(false);
-  _hideKillAction.setEnabled(false);
-  _hideQuitAction.setEnabled(false);
-  _hideModeAction.setEnabled(false);
-
-  showChannelList.setIcon(QIcon(":/16x16/actions/oxygen/16x16/actions/format-list-unordered.png"));
-
-  connect(this, SIGNAL(collapsed(const QModelIndex &)), this, SLOT(on_collapse(const QModelIndex &)));
-  connect(this, SIGNAL(expanded(const QModelIndex &)), this, SLOT(on_expand(const QModelIndex &)));
+  connect(this, SIGNAL(collapsed(const QModelIndex &)), SLOT(storeExpandedState(const QModelIndex &)));
+  connect(this, SIGNAL(expanded(const QModelIndex &)), SLOT(storeExpandedState(const QModelIndex &)));
 
   setSelectionMode(QAbstractItemView::ExtendedSelection);
+
+  QAbstractItemDelegate *oldDelegate = itemDelegate();
+  BufferViewDelegate *tristateDelegate = new BufferViewDelegate(this);
+  setItemDelegate(tristateDelegate);
+  delete oldDelegate;
+
+  UiStyleSettings s("QtUiStyle/Fonts"); // li'l dirty here, but fonts are stored in QtUiStyle :/
+  s.notify("BufferView", this, SLOT(setCustomFont(QVariant)));
+  setCustomFont(s.value("BufferView", QFont()));
 }
 
 void BufferView::init() {
-  setIndentation(10);
   header()->setContextMenuPolicy(Qt::ActionsContextMenu);
   hideColumn(1);
   hideColumn(2);
+  setIndentation(10);
   expandAll();
 
   setAnimated(true);
@@ -104,18 +83,21 @@ void BufferView::init() {
 
   setSortingEnabled(true);
   sortByColumn(0, Qt::AscendingOrder);
-#ifndef Q_WS_QWS
-  // this is a workaround to not join channels automatically... we need a saner way to navigate for qtopia anyway though,
-  // such as mark first, activate at second click...
-  connect(this, SIGNAL(activated(QModelIndex)), this, SLOT(joinChannel(QModelIndex)));
+
+  // activated() fails on X11 and Qtopia at least
+#if defined Q_WS_QWS || defined Q_WS_X11
+  disconnect(this, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(joinChannel(QModelIndex)));
+  connect(this, SIGNAL(doubleClicked(QModelIndex)), SLOT(joinChannel(QModelIndex)));
 #else
-  connect(this, SIGNAL(doubleClicked(QModelIndex)), this, SLOT(joinChannel(QModelIndex)));  // Qtopia uses single click for activation
+  // afaik this is better on Mac and Windows
+  disconnect(this, SIGNAL(activated(QModelIndex)), this, SLOT(joinChannel(QModelIndex)));
+  connect(this, SIGNAL(activated(QModelIndex)), SLOT(joinChannel(QModelIndex)));
 #endif
 }
 
 void BufferView::setModel(QAbstractItemModel *model) {
   delete selectionModel();
-  
+
   QTreeView::setModel(model);
   init();
   // remove old Actions
@@ -139,7 +121,8 @@ void BufferView::setModel(QAbstractItemModel *model) {
     connect(showSection, SIGNAL(toggled(bool)), this, SLOT(toggleHeader(bool)));
     header()->addAction(showSection);
   }
-  
+
+  connect(model, SIGNAL(layoutChanged()), this, SLOT(on_layoutChanged()));
 }
 
 void BufferView::setFilteredModel(QAbstractItemModel *model_, BufferViewConfig *config) {
@@ -168,20 +151,20 @@ void BufferView::setFilteredModel(QAbstractItemModel *model_, BufferViewConfig *
 void BufferView::setSelectionModel(QItemSelectionModel *selectionModel) {
   if(QTreeView::selectionModel())
     disconnect(selectionModel, SIGNAL(currentChanged(QModelIndex, QModelIndex)),
-	       model(), SIGNAL(checkPreviousCurrentForRemoval(QModelIndex, QModelIndex)));
-    
+               model(), SIGNAL(checkPreviousCurrentForRemoval(QModelIndex, QModelIndex)));
+
   QTreeView::setSelectionModel(selectionModel);
   BufferViewFilter *filter = qobject_cast<BufferViewFilter *>(model());
   if(filter) {
     connect(selectionModel, SIGNAL(currentChanged(QModelIndex, QModelIndex)),
-	    filter, SLOT(checkPreviousCurrentForRemoval(QModelIndex, QModelIndex)));
+            filter, SLOT(checkPreviousCurrentForRemoval(QModelIndex, QModelIndex)));
   }
 }
 
 void BufferView::setConfig(BufferViewConfig *config) {
   if(_config == config)
     return;
-  
+
   if(_config) {
     disconnect(_config, 0, this, 0);
   }
@@ -191,24 +174,34 @@ void BufferView::setConfig(BufferViewConfig *config) {
     connect(config, SIGNAL(networkIdSet(const NetworkId &)), this, SLOT(setRootIndexForNetworkId(const NetworkId &)));
     setRootIndexForNetworkId(config->networkId());
   } else {
+    setIndentation(10);
     setRootIndex(QModelIndex());
   }
 }
 
 void BufferView::setRootIndexForNetworkId(const NetworkId &networkId) {
   if(!networkId.isValid() || !model()) {
+    setIndentation(10);
     setRootIndex(QModelIndex());
   } else {
+    setIndentation(5);
     int networkCount = model()->rowCount();
     QModelIndex child;
     for(int i = 0; i < networkCount; i++) {
       child = model()->index(i, 0);
       if(networkId == model()->data(child, NetworkModel::NetworkIdRole).value<NetworkId>())
-	setRootIndex(child);
+        setRootIndex(child);
     }
   }
 }
 
+void BufferView::setCustomFont(const QVariant &v) {
+  QFont font = v.value<QFont>();
+  if(font.family().isEmpty())
+    font = QApplication::font();
+  setFont(font);
+}
+
 void BufferView::joinChannel(const QModelIndex &index) {
   BufferInfo::Type bufferType = (BufferInfo::Type)index.data(NetworkModel::BufferTypeRole).value<int>();
 
@@ -216,7 +209,7 @@ void BufferView::joinChannel(const QModelIndex &index) {
     return;
 
   BufferInfo bufferInfo = index.data(NetworkModel::BufferInfoRole).value<BufferInfo>();
-  
+
   Client::userInput(bufferInfo, QString("/JOIN %1").arg(bufferInfo.bufferName()));
 }
 
@@ -228,6 +221,46 @@ void BufferView::keyPressEvent(QKeyEvent *event) {
   QTreeView::keyPressEvent(event);
 }
 
+void BufferView::dropEvent(QDropEvent *event) {
+  QModelIndex index = indexAt(event->pos());
+
+  QRect indexRect = visualRect(index);
+  QPoint cursorPos = event->pos();
+
+  // check if we're really _on_ the item and not indicating a move to just above or below the item
+  const int margin = 2;
+  if(cursorPos.y() - indexRect.top() < margin
+     || indexRect.bottom() - cursorPos.y() < margin)
+    return QTreeView::dropEvent(event);
+
+  QList< QPair<NetworkId, BufferId> > bufferList = Client::networkModel()->mimeDataToBufferList(event->mimeData());
+  if(bufferList.count() != 1)
+    return QTreeView::dropEvent(event);
+
+  NetworkId networkId = bufferList[0].first;
+  BufferId bufferId2 = bufferList[0].second;
+
+  if(index.data(NetworkModel::ItemTypeRole) != NetworkModel::BufferItemType)
+    return QTreeView::dropEvent(event);
+
+  if(index.data(NetworkModel::BufferTypeRole) != BufferInfo::QueryBuffer)
+    return QTreeView::dropEvent(event);
+
+  if(index.data(NetworkModel::NetworkIdRole).value<NetworkId>() != networkId)
+    return QTreeView::dropEvent(event);
+
+  BufferId bufferId1 = index.data(NetworkModel::BufferIdRole).value<BufferId>();
+  if(bufferId1 == bufferId2)
+    return QTreeView::dropEvent(event);
+
+  int res = QMessageBox::question(0, tr("Merge buffers permanently?"),
+				  tr("Do you want to merge the buffer \"%1\" permanently into buffer \"%2\"?\n This cannot be reversed!").arg(Client::networkModel()->bufferName(bufferId2)).arg(Client::networkModel()->bufferName(bufferId1)),
+				  QMessageBox::Yes|QMessageBox::No, QMessageBox::No);
+  if(res == QMessageBox::Yes) {
+    Client::mergeBuffersPermanently(bufferId1, bufferId2);
+  }
+}
+
 void BufferView::removeSelectedBuffers(bool permanently) {
   if(!config())
     return;
@@ -243,7 +276,7 @@ void BufferView::removeSelectedBuffers(bool permanently) {
       continue;
 
     removedRows << bufferId;
-    
+
     if(permanently)
       config()->requestRemoveBufferPermanently(bufferId);
     else
@@ -251,17 +284,21 @@ void BufferView::removeSelectedBuffers(bool permanently) {
   }
 }
 
-void BufferView::rowsInserted(const QModelIndex & parent, int start, int end) {
+void BufferView::rowsInserted(const QModelIndex &parent, int start, int end) {
   QTreeView::rowsInserted(parent, start, end);
 
   // ensure that newly inserted network nodes are expanded per default
   if(parent.data(NetworkModel::ItemTypeRole) != NetworkModel::NetworkItemType)
     return;
-  
-  if(model()->rowCount(parent) == 1 && parent.data(NetworkModel::ItemActiveRole) == true) {
-    // without updating the parent the expand will have no effect... Qt Bug?
-    update(parent);
-    expand(parent);
+
+  setExpandedState(parent);
+}
+
+void BufferView::on_layoutChanged() {
+  int numNets = model()->rowCount(QModelIndex());
+  for(int row = 0; row < numNets; row++) {
+    QModelIndex networkIdx = model()->index(row, 0, QModelIndex());
+    setExpandedState(networkIdx);
   }
 }
 
@@ -280,44 +317,54 @@ void BufferView::on_configChanged() {
     if(!networkId.isValid())
       continue;
 
-    update(networkIdx);
-
-    bool expandNetwork = false;
-    if(_expandedState.contains(networkId))
-      expandNetwork = _expandedState[networkId];
-    else
-      expandNetwork = model()->data(networkIdx, NetworkModel::ItemActiveRole).toBool();
-
-    if(expandNetwork)
-      expand(networkIdx);
-    else
-      collapse(networkIdx);
+    setExpandedState(networkIdx);
   }
 
-  // update selection to current one
-  MappedSelectionModel *mappedSelectionModel = qobject_cast<MappedSelectionModel *>(selectionModel());
-  if(!config() || !mappedSelectionModel)
+  if(config()) {
+    // update selection to current one
+    Client::bufferModel()->synchronizeView(this);
+  }
+}
+
+void BufferView::storeExpandedState(const QModelIndex &networkIdx) {
+  NetworkId networkId = model()->data(networkIdx, NetworkModel::NetworkIdRole).value<NetworkId>();
+
+  int oldState = 0;
+  if(isExpanded(networkIdx))
+    oldState |= WasExpanded;
+  if(model()->data(networkIdx, NetworkModel::ItemActiveRole).toBool())
+    oldState |= WasActive;
+
+  _expandedState[networkId] = oldState;
+}
+
+void BufferView::setExpandedState(const QModelIndex &networkIdx) {
+  if(model()->data(networkIdx, NetworkModel::ItemTypeRole) != NetworkModel::NetworkItemType)
     return;
 
-  mappedSelectionModel->mappedSetCurrentIndex(Client::bufferModel()->standardSelectionModel()->currentIndex(), QItemSelectionModel::Current);
-  mappedSelectionModel->mappedSelect(Client::bufferModel()->standardSelectionModel()->selection(), QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows);
-}
+  if(model()->rowCount(networkIdx) == 0)
+    return;
 
-void BufferView::on_collapse(const QModelIndex &index) {
-  storeExpandedState(index.data(NetworkModel::NetworkIdRole).value<NetworkId>(), false);
-}
+  NetworkId networkId = model()->data(networkIdx, NetworkModel::NetworkIdRole).value<NetworkId>();
 
-void BufferView::on_expand(const QModelIndex &index) {
-  storeExpandedState(index.data(NetworkModel::NetworkIdRole).value<NetworkId>(), true);
-}
+  bool networkActive = model()->data(networkIdx, NetworkModel::ItemActiveRole).toBool();
+  bool expandNetwork = networkActive;
+  if(_expandedState.contains(networkId)) {
+    int oldState = _expandedState[networkId];
+    if((bool)(oldState & WasActive) == networkActive)
+      expandNetwork = (bool)(oldState & WasExpanded);
+  }
 
-void BufferView::storeExpandedState(NetworkId networkId, bool expanded) {
-  _expandedState[networkId] = expanded;
+  if(expandNetwork != isExpanded(networkIdx)) {
+    update(networkIdx);
+    setExpanded(networkIdx, expandNetwork);
+  }
+  storeExpandedState(networkIdx); // this call is needed to keep track of the isActive state
 }
 
 void BufferView::dataChanged(const QModelIndex &topLeft, const QModelIndex &bottomRight) {
   QTreeView::dataChanged(topLeft, bottomRight);
-  
+
   // determine how many items have been changed and if any of them is a networkitem
   // which just swichted from active to inactive or vice versa
   if(topLeft.data(NetworkModel::ItemTypeRole) != NetworkModel::NetworkItemType)
@@ -325,15 +372,7 @@ void BufferView::dataChanged(const QModelIndex &topLeft, const QModelIndex &bott
 
   for(int i = topLeft.row(); i <= bottomRight.row(); i++) {
     QModelIndex networkIdx = topLeft.sibling(i, 0);
-    if(model()->rowCount(networkIdx) == 0)
-      continue;
-
-    bool isActive = networkIdx.data(NetworkModel::ItemActiveRole).toBool();
-#ifdef SPUTDEV
-    if(isExpanded(networkIdx) != isActive) setExpanded(networkIdx, true);
-#else
-    if(isExpanded(networkIdx) != isActive) setExpanded(networkIdx, isActive);
-#endif
+    setExpandedState(networkIdx);
   }
 }
 
@@ -342,191 +381,57 @@ void BufferView::toggleHeader(bool checked) {
   header()->setSectionHidden((action->property("column")).toInt(), !checked);
 }
 
-bool BufferView::checkRequirements(const QModelIndex &index, ItemActiveStates requiredActiveState) {
-  if(!index.isValid())
-    return false;
-
-  ItemActiveStates isActive = index.data(NetworkModel::ItemActiveRole).toBool()
-    ? ActiveState
-    : InactiveState;
-
-  if(!(isActive & requiredActiveState))
-    return false;
-
-  return true;
-}
-
-void BufferView::addItemToMenu(QAction &action, QMenu &menu, const QModelIndex &index, ItemActiveStates requiredActiveState) {
-  if(checkRequirements(index, requiredActiveState)) {
-    menu.addAction(&action);
-    action.setVisible(true);
-  } else {
-    action.setVisible(false);
-  }
-}
-
-void BufferView::addItemToMenu(QAction &action, QMenu &menu, bool condition) {
-  if(condition) {
-    menu.addAction(&action);
-    action.setVisible(true);
-  } else {
-    action.setVisible(false);
-  }
-}
-
-
-void BufferView::addItemToMenu(QMenu &subMenu, QMenu &menu, const QModelIndex &index, ItemActiveStates requiredActiveState) {
-  if(checkRequirements(index, requiredActiveState)) {
-    menu.addMenu(&subMenu);
-    subMenu.setVisible(true);
-  } else {
-    subMenu.setVisible(false);
-  }
-}
-
-void BufferView::addSeparatorToMenu(QMenu &menu, const QModelIndex &index, ItemActiveStates requiredActiveState) {
-  if(checkRequirements(index, requiredActiveState)) {
-    menu.addSeparator();
-  }
-}
-
-QMenu *BufferView::createHideEventsSubMenu(QMenu &menu) {
-  // QMenu *hideEventsMenu = new QMenu(tr("Hide Events"), &menu);
-  QMenu *hideEventsMenu = menu.addMenu(tr("Hide Events"));
-  hideEventsMenu->addAction(&_hideJoinAction);
-  hideEventsMenu->addAction(&_hidePartAction);
-  hideEventsMenu->addAction(&_hideKillAction);
-  hideEventsMenu->addAction(&_hideQuitAction);
-  hideEventsMenu->addAction(&_hideModeAction);
-  return hideEventsMenu;
-}
-
 void BufferView::contextMenuEvent(QContextMenuEvent *event) {
   QModelIndex index = indexAt(event->pos());
   if(!index.isValid())
     index = rootIndex();
-  if(!index.isValid())
-    return;
 
-  const Network *network = Client::network(index.data(NetworkModel::NetworkIdRole).value<NetworkId>());
-  Q_CHECK_PTR(network);
-
-  QIcon connectionStateIcon;
-  if(network) {
-    if(network->connectionState() == Network::Initialized) {
-      connectionStateIcon = QIcon(":/22x22/actions/network-connect");
-    } else if(network->connectionState() == Network::Disconnected) {
-      connectionStateIcon = QIcon(":/22x22/actions/network-disconnect");
-    } else {
-      connectionStateIcon = QIcon(":/22x22/actions/gear");
-    }
-  }
-  
   QMenu contextMenu(this);
-  NetworkModel::itemType itemType = static_cast<NetworkModel::itemType>(index.data(NetworkModel::ItemTypeRole).toInt());
-  
-  switch(itemType) {
-  case NetworkModel::NetworkItemType:
-    showChannelList.setData(index.data(NetworkModel::NetworkIdRole));
-    _disconnectNetAction.setIcon(connectionStateIcon);
-    _connectNetAction.setIcon(connectionStateIcon);
-    addItemToMenu(showChannelList, contextMenu, index, ActiveState);
-    addItemToMenu(_disconnectNetAction, contextMenu, network->connectionState() != Network::Disconnected);
-    addItemToMenu(_connectNetAction, contextMenu, network->connectionState() == Network::Disconnected);
-    addSeparatorToMenu(contextMenu, index, ActiveState);
-    addItemToMenu(_joinChannelAction, contextMenu, index, ActiveState);
-    break;
-  case NetworkModel::BufferItemType:
-    {
-      BufferInfo bufferInfo = index.data(NetworkModel::BufferInfoRole).value<BufferInfo>();
-      switch(bufferInfo.type()) {
-      case BufferInfo::ChannelBuffer:
-	addItemToMenu(_joinBufferAction, contextMenu, index, InactiveState);
-	addItemToMenu(_partBufferAction, contextMenu, index, ActiveState);
-	addItemToMenu(_hideBufferTemporarilyAction, contextMenu, (bool)config());
-	addItemToMenu(_hideBufferPermanentlyAction, contextMenu, (bool)config());
-	addItemToMenu(_removeBufferAction, contextMenu, index, InactiveState);
-	createHideEventsSubMenu(contextMenu);
-	addItemToMenu(_ignoreListAction, contextMenu);
-	break;
-      case BufferInfo::QueryBuffer:
-	addItemToMenu(_hideBufferTemporarilyAction, contextMenu, (bool)config());
-	addItemToMenu(_hideBufferPermanentlyAction, contextMenu, (bool)config());
-	addItemToMenu(_removeBufferAction, contextMenu);
-	createHideEventsSubMenu(contextMenu);
-	break;
-      default:
-	addItemToMenu(_hideBufferTemporarilyAction, contextMenu, (bool)config());
-	addItemToMenu(_hideBufferPermanentlyAction, contextMenu, (bool)config());
-	break;
+
+  if(index.isValid()) {
+    addActionsToMenu(&contextMenu, index);
+  }
+
+  addFilterActions(&contextMenu, index);
+
+  if(!contextMenu.actions().isEmpty())
+    contextMenu.exec(QCursor::pos());
+}
+
+void BufferView::addActionsToMenu(QMenu *contextMenu, const QModelIndex &index) {
+  QModelIndexList indexList = selectedIndexes();
+  // make sure the item we clicked on is first
+  indexList.removeAll(index);
+  indexList.prepend(index);
+
+  GraphicalUi::contextMenuActionProvider()->addActions(contextMenu, indexList, this, "menuActionTriggered", (bool)config());
+}
+
+void BufferView::addFilterActions(QMenu *contextMenu, const QModelIndex &index) {
+  BufferViewFilter *filter = qobject_cast<BufferViewFilter *>(model());
+  if(filter) {
+    QList<QAction *> filterActions = filter->actions(index);
+    if(!filterActions.isEmpty()) {
+      contextMenu->addSeparator();
+      foreach(QAction *action, filterActions) {
+	contextMenu->addAction(action);
       }
     }
-    break;
-  default:
-    return;
-  }
-  
-  if(contextMenu.actions().isEmpty())
-    return;
-  QAction *result = contextMenu.exec(QCursor::pos());
-  
-  // Handle Result
-  if(network && result == &_connectNetAction) {
-    network->requestConnect();
-    return;
   }
+}
 
-  if(network && result == &_disconnectNetAction) {
-    network->requestDisconnect();
-    return;
+void BufferView::menuActionTriggered(QAction *result) {
+  ContextMenuActionProvider::ActionType type = (ContextMenuActionProvider::ActionType)result->data().toInt();
+  switch(type) {
+    case ContextMenuActionProvider::HideBufferTemporarily:
+      removeSelectedBuffers();
+      break;
+    case ContextMenuActionProvider::HideBufferPermanently:
+      removeSelectedBuffers(true);
+      break;
+    default:
+      return;
   }
-
-  if(result == &_joinChannelAction) {
-    // FIXME no QInputDialog in Qtopia
-#ifndef Q_WS_QWS
-    bool ok;
-    QString channelName = QInputDialog::getText(this, tr("Join Channel"), tr("Input channel name:"), QLineEdit::Normal, QString(), &ok);
-    if(ok && !channelName.isEmpty()) {
-      Client::instance()->userInput(BufferInfo::fakeStatusBuffer(index.data(NetworkModel::NetworkIdRole).value<NetworkId>()), QString("/J %1").arg(channelName));
-    }
-#endif
-    return;
-  }
-
-  if(result == &_joinBufferAction) {
-    BufferInfo bufferInfo = index.data(NetworkModel::BufferInfoRole).value<BufferInfo>();
-    Client::instance()->userInput(bufferInfo, QString("/JOIN %1").arg(bufferInfo.bufferName()));
-    return;
-  }
-
-  if(result == &_partBufferAction) {
-    BufferInfo bufferInfo = index.data(NetworkModel::BufferInfoRole).value<BufferInfo>();
-    Client::instance()->userInput(bufferInfo, QString("/PART"));
-    return;
-  }
-  
-  if(result == &_hideBufferTemporarilyAction) {
-    removeSelectedBuffers();
-    return;
-  }
-
-  if(result == &_hideBufferPermanentlyAction) {
-    removeSelectedBuffers(true);
-    return;
-  }
-
-  if(result == &_removeBufferAction) {
-    BufferInfo bufferInfo = index.data(NetworkModel::BufferInfoRole).value<BufferInfo>();
-    int res = QMessageBox::question(this, tr("Remove buffer permanently?"),
-                                    tr("Do you want to delete the buffer \"%1\" permanently? This will delete all related data, including all backlog "
-                                       "data, from the core's database!").arg(bufferInfo.bufferName()),
-                                        QMessageBox::Yes|QMessageBox::No, QMessageBox::No);
-    if(res == QMessageBox::Yes) {
-      Client::removeBuffer(bufferInfo.bufferId());
-    }
-    return;
-  }
-
 }
 
 void BufferView::wheelEvent(QWheelEvent* event) {
@@ -553,13 +458,12 @@ void BufferView::wheelEvent(QWheelEvent* event) {
       }
   selectionModel()->setCurrentIndex( resultingIndex, QItemSelectionModel::ClearAndSelect | QItemSelectionModel::Rows );
   selectionModel()->select( resultingIndex, QItemSelectionModel::ClearAndSelect );
-  
-}
 
+}
 
 QSize BufferView::sizeHint() const {
   return QTreeView::sizeHint();
-  
+
   if(!model())
     return QTreeView::sizeHint();
 
@@ -574,6 +478,110 @@ QSize BufferView::sizeHint() const {
   return QSize(columnSize, 50);
 }
 
+
+// ****************************************
+//  BufferViewDelgate
+// ****************************************
+class ColorsChangedEvent : public QEvent {
+public:
+  ColorsChangedEvent() : QEvent(QEvent::User) {};
+};
+
+BufferViewDelegate::BufferViewDelegate(QObject *parent)
+  : QStyledItemDelegate(parent),
+    _updateColors(false)
+{
+  loadColors();
+
+  UiSettings s("QtUiStyle/Colors");
+  s.notify("inactiveActivityFG", this, SLOT(colorsChanged()));
+  s.notify("noActivityFG", this, SLOT(colorsChanged()));
+  s.notify("highlightActivityFG", this, SLOT(colorsChanged()));
+  s.notify("newMessageActivityFG", this, SLOT(colorsChanged()));
+  s.notify("otherActivityFG", this, SLOT(colorsChanged()));
+}
+
+void BufferViewDelegate::colorsChanged() {
+  // avoid mutliple unneded reloads of all colors
+  if(_updateColors)
+    return;
+  _updateColors = true;
+  QCoreApplication::postEvent(this, new ColorsChangedEvent());
+}
+
+void BufferViewDelegate::customEvent(QEvent *event) {
+  if(event->type() != QEvent::User)
+    return;
+
+  loadColors();
+  _updateColors = false;
+
+  event->accept();
+}
+
+void BufferViewDelegate::loadColors() {
+  UiSettings s("QtUiStyle/Colors");
+  _FgColorInactiveActivity = s.value("inactiveActivityFG", QVariant(QColor(Qt::gray))).value<QColor>();
+  _FgColorNoActivity = s.value("noActivityFG", QVariant(QColor(Qt::black))).value<QColor>();
+  _FgColorHighlightActivity = s.value("highlightActivityFG", QVariant(QColor(Qt::magenta))).value<QColor>();
+  _FgColorNewMessageActivity = s.value("newMessageActivityFG", QVariant(QColor(Qt::green))).value<QColor>();
+  _FgColorOtherActivity = s.value("otherActivityFG", QVariant(QColor(Qt::darkGreen))).value<QColor>();
+}
+
+bool BufferViewDelegate::editorEvent(QEvent *event, QAbstractItemModel *model, const QStyleOptionViewItem &option, const QModelIndex &index) {
+  if(event->type() != QEvent::MouseButtonRelease)
+    return QStyledItemDelegate::editorEvent(event, model, option, index);
+
+  if(!(model->flags(index) & Qt::ItemIsUserCheckable))
+    return QStyledItemDelegate::editorEvent(event, model, option, index);
+
+  QVariant value = index.data(Qt::CheckStateRole);
+  if(!value.isValid())
+    return QStyledItemDelegate::editorEvent(event, model, option, index);
+
+  QStyleOptionViewItemV4 viewOpt(option);
+  initStyleOption(&viewOpt, index);
+
+  QRect checkRect = viewOpt.widget->style()->subElementRect(QStyle::SE_ItemViewItemCheckIndicator, &viewOpt, viewOpt.widget);
+  QMouseEvent *me = static_cast<QMouseEvent*>(event);
+
+  if(me->button() != Qt::LeftButton || !checkRect.contains(me->pos()))
+    return QStyledItemDelegate::editorEvent(event, model, option, index);
+
+  Qt::CheckState state = static_cast<Qt::CheckState>(value.toInt());
+  if(state == Qt::Unchecked)
+    state = Qt::PartiallyChecked;
+  else if(state == Qt::PartiallyChecked)
+    state = Qt::Checked;
+  else
+    state = Qt::Unchecked;
+  model->setData(index, state, Qt::CheckStateRole);
+  return true;
+}
+
+void BufferViewDelegate::initStyleOption(QStyleOptionViewItem *option, const QModelIndex &index) const {
+  QStyledItemDelegate::initStyleOption(option, index);
+
+  if(!index.isValid())
+    return;
+
+  BufferInfo::ActivityLevel activity = (BufferInfo::ActivityLevel)index.data(NetworkModel::BufferActivityRole).toInt();
+
+  QColor fgColor = _FgColorNoActivity;
+  if(activity & BufferInfo::Highlight) {
+    fgColor = _FgColorHighlightActivity;
+  } else if(activity & BufferInfo::NewMessage) {
+    fgColor = _FgColorNewMessageActivity;
+  } else if(activity & BufferInfo::OtherActivity) {
+    fgColor = _FgColorOtherActivity;
+  } else if(!index.data(NetworkModel::ItemActiveRole).toBool() || index.data(NetworkModel::UserAwayRole).toBool()) {
+    fgColor = _FgColorInactiveActivity;
+  }
+
+  option->palette.setColor(QPalette::Text, fgColor);
+}
+
+
 // ==============================
 //  BufferView Dock
 // ==============================
@@ -586,14 +594,6 @@ BufferViewDock::BufferViewDock(BufferViewConfig *config, QWidget *parent)
   connect(config, SIGNAL(bufferViewNameSet(const QString &)), this, SLOT(bufferViewRenamed(const QString &)));
 }
 
-BufferViewDock::BufferViewDock(QWidget *parent)
-  : QDockWidget(tr("All Buffers"), parent)
-{
-  setObjectName("BufferViewDock--1");
-  toggleViewAction()->setData((int)-1);
-  setAllowedAreas(Qt::RightDockWidgetArea|Qt::LeftDockWidgetArea);
-}
-
 void BufferViewDock::bufferViewRenamed(const QString &newName) {
   setWindowTitle(newName);
   toggleViewAction()->setText(newName);
